# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'loglayout.ui',
# licensing of 'loglayout.ui' applies.
#
# Created: Sun Nov  3 00:30:05 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_LogLayout(object):
    def setupUi(self, LogLayout):
        LogLayout.setObjectName("LogLayout")
        LogLayout.resize(400, 300)
        self.verticalLayout = QtWidgets.QVBoxLayout(LogLayout)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea = QtWidgets.QScrollArea(LogLayout)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.logContents = QtWidgets.QWidget()
        self.logContents.setGeometry(QtCore.QRect(0, 0, 380, 280))
        self.logContents.setObjectName("logContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.logContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.scrollArea.setWidget(self.logContents)
        self.verticalLayout.addWidget(self.scrollArea)

        self.retranslateUi(LogLayout)
        QtCore.QMetaObject.connectSlotsByName(LogLayout)

    def retranslateUi(self, LogLayout):
        LogLayout.setWindowTitle(
            QtWidgets.QApplication.translate("LogLayout", "LogLayout", None, -1)
        )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'signalwidget.ui',
# licensing of 'signalwidget.ui' applies.
#
# Created: Sun Nov  3 00:30:06 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_SignalWidget(object):
    def setupUi(self, SignalWidget):
        SignalWidget.setObjectName("SignalWidget")
        SignalWidget.resize(270, 52)
        self.horizontalLayout = QtWidgets.QHBoxLayout(SignalWidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.name = QtWidgets.QLabel(SignalWidget)
        self.name.setText("")
        self.name.setObjectName("name")
        self.horizontalLayout.addWidget(self.name)
        self.start = QtWidgets.QLabel(SignalWidget)
        self.start.setText("")
        self.start.setObjectName("start")
        self.horizontalLayout.addWidget(self.start)
        self.length = QtWidgets.QLabel(SignalWidget)
        self.length.setText("")
        self.length.setObjectName("length")
        self.horizontalLayout.addWidget(self.length)
        self.signalDetailsButton = QtWidgets.QPushButton(SignalWidget)
        self.signalDetailsButton.setCheckable(True)
        self.signalDetailsButton.setObjectName("signalDetailsButton")
        self.horizontalLayout.addWidget(self.signalDetailsButton)

        self.retranslateUi(SignalWidget)
        QtCore.QMetaObject.connectSlotsByName(SignalWidget)

    def retranslateUi(self, SignalWidget):
        SignalWidget.setWindowTitle(
            QtWidgets.QApplication.translate("SignalWidget", "SignalWidget", None, -1)
        )
        self.signalDetailsButton.setText(
            QtWidgets.QApplication.translate("SignalWidget", "Details", None, -1)
        )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'cmddetails.ui',
# licensing of 'cmddetails.ui' applies.
#
# Created: Sun Nov  3 00:30:04 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_CmdDetails(object):
    def setupUi(self, CmdDetails):
        CmdDetails.setObjectName("CmdDetails")
        CmdDetails.resize(462, 428)
        CmdDetails.setMinimumSize(QtCore.QSize(300, 0))
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(CmdDetails)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.groupBox = QtWidgets.QGroupBox(CmdDetails)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.nameLabel = QtWidgets.QLabel(self.groupBox)
        self.nameLabel.setObjectName("nameLabel")
        self.horizontalLayout.addWidget(self.nameLabel)
        self.nameEdit = QtWidgets.QLineEdit(self.groupBox)
        self.nameEdit.setObjectName("nameEdit")
        self.horizontalLayout.addWidget(self.nameEdit)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.commentLabel = QtWidgets.QLabel(self.groupBox)
        self.commentLabel.setObjectName("commentLabel")
        self.horizontalLayout_3.addWidget(self.commentLabel)
        self.commentEdit = QtWidgets.QLineEdit(self.groupBox)
        self.commentEdit.setObjectName("commentEdit")
        self.horizontalLayout_3.addWidget(self.commentEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.n_argsLabel = QtWidgets.QLabel(self.groupBox)
        self.n_argsLabel.setObjectName("n_argsLabel")
        self.horizontalLayout_2.addWidget(self.n_argsLabel)
        self.n_argsEdit = QtWidgets.QLineEdit(self.groupBox)
        self.n_argsEdit.setObjectName("n_argsEdit")
        self.horizontalLayout_2.addWidget(self.n_argsEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.idLabel = QtWidgets.QLabel(self.groupBox)
        self.idLabel.setObjectName("idLabel")
        self.horizontalLayout_4.addWidget(self.idLabel)
        self.idEdit = QtWidgets.QLineEdit(self.groupBox)
        self.idEdit.setObjectName("idEdit")
        self.horizontalLayout_4.addWidget(self.idEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self._CmdArgContents = QtWidgets.QWidget()
        self._CmdArgContents.setGeometry(QtCore.QRect(0, 0, 408, 182))
        self._CmdArgContents.setObjectName("_CmdArgContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self._CmdArgContents)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.CmdArgContents = QtWidgets.QVBoxLayout()
        self.CmdArgContents.setObjectName("CmdArgContents")
        self.verticalLayout_3.addLayout(self.CmdArgContents)
        self.scrollArea.setWidget(self._CmdArgContents)
        self.verticalLayout.addWidget(self.scrollArea)
        self.addArgButton = QtWidgets.QPushButton(self.groupBox)
        self.addArgButton.setObjectName("addArgButton")
        self.verticalLayout.addWidget(self.addArgButton)
        self.verticalLayout_4.addLayout(self.verticalLayout)
        self.deleteCmdButton = QtWidgets.QPushButton(self.groupBox)
        self.deleteCmdButton.setObjectName("deleteCmdButton")
        self.verticalLayout_4.addWidget(self.deleteCmdButton)
        self.horizontalLayout_5.addWidget(self.groupBox)
        self.argDetailsLayout = QtWidgets.QVBoxLayout()
        self.argDetailsLayout.setObjectName("argDetailsLayout")
        self.horizontalLayout_5.addLayout(self.argDetailsLayout)

        self.retranslateUi(CmdDetails)
        QtCore.QMetaObject.connectSlotsByName(CmdDetails)

    def retranslateUi(self, CmdDetails):
        CmdDetails.setWindowTitle(
            QtWidgets.QApplication.translate("CmdDetails", "CmdDetails", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("CmdDetails", "Command", None, -1)
        )
        self.nameLabel.setText(
            QtWidgets.QApplication.translate("CmdDetails", "name", None, -1)
        )
        self.commentLabel.setText(
            QtWidgets.QApplication.translate("CmdDetails", "comment", None, -1)
        )
        self.n_argsLabel.setText(
            QtWidgets.QApplication.translate("CmdDetails", "n_args", None, -1)
        )
        self.idLabel.setText(
            QtWidgets.QApplication.translate("CmdDetails", "id", None, -1)
        )
        self.addArgButton.setText(
            QtWidgets.QApplication.translate("CmdDetails", "Add Arg", None, -1)
        )
        self.deleteCmdButton.setText(
            QtWidgets.QApplication.translate("CmdDetails", "Delete", None, -1)
        )



import os
from copy import deepcopy
from pprint import pprint
from datetime import datetime

from ww import f

from .spec import Signal, Message, Device, Common, Spec, make_sid


def date():
    return str(datetime.now())


def decode_signature(device, message, signal):
    id = make_sid(device.id, message.id)
    return f("decode_{id}_{signal.name}")


def decode_signal_sig(device, message, signal):
    return f("{signal.type} {decode_signature(device, message, signal)} (CANdata msg);")


def msg_struct_sig(device, message):
    return f("msg_{device.name}_{message.name}_t")


def device_struct_sig(device):
    return f("dev_{device.name}_t")


def spec_struct_sig(spec):
    return "can_t"


def build_msg_sig(device):
    for message in device.msgs.values():
        message.type = msg_struct_sig(device, message)


def multiplexor(signal, message):
    if signal.mux == "" or signal.mux == None:
        signal.mux_str = ""
        return

    sig = message.get_signal(signal.mux)
    signal.mux_str = f("[{sig.max_value - sig.min_value + 1}]")


def dst_type_decide(signal):
    if signal.scale != 1:
        if signal.length == 32:
            return "float"
        elif signal.length == 64:
            return "double"

    if signal.type == "signed":
        if signal.length <= 8:
            return "int8_t"
        elif signal.length <= 16:
            return "int16_t"
        elif signal.length <= 32:
            return "int32_t"
        elif signal.length <= 64:
            return "int64_t"

    if signal.length <= 8:
        return "uint8_t"
    elif signal.length <= 16:
        return "uint16_t"
    elif signal.length <= 32:
        return "uint32_t"
    elif signal.length <= 64:
        return "uint64_t"


def build_devices(spec, device, tpl):
    for message in device.msgs.values():
        message.multiplexor = ""
        message.mux_count = 0
        message.full_id = make_sid(device.id, message.id)
        for signal in message.signals.values():
            signal.dst_type = dst_type_decide(signal)
            multiplexor(signal, message)
            if signal.mux != "":
                message.multiplexor = signal.mux
            if signal.mux_count != 1:
                message.mux_count = signal.mux_count

    build_msg_sig(device)

    device.signature = device_struct_sig(device)

    h = tpl.tpl["h.jinja"].render(device=device, date=date())
    c = tpl.tpl["c.jinja"].render(
        device=device,
        f_msgs=[msg for msg in device.msgs.values() if msg.frequency != 0],
        date=date(),
    )

    return device.name, h, c


def build_can_ids(spec, tpl):
    for device in spec.devices.values():
        device.signature = device_struct_sig(device)

    spec.signature = spec_struct_sig(spec)

    build_msg_sig(spec.common)
    spec.common.signature = device_struct_sig(spec.common)

    logs = list(sorted(spec.logs.values(), key=lambda x: x.id))
    can_ids_h = tpl.tpl["can_ids_h.jinja"].render(
        devices=sorted(spec.devices.values(), key=lambda x: x.id),
        spec=spec,
        logs=logs,
        date=date(),
    )

    can_ids_c = tpl.tpl["can_ids_c.jinja"].render(
        devices=sorted(spec.devices.values(), key=lambda x: x.id),
        spec=spec,
        logs=logs,
        date=date(),
    )

    return can_ids_c, can_ids_h


def build_common(spec, tpl):
    for message in spec.common.msgs.values():
        sigs = message.signals
        for signal in message.signals.values():
            signal.dst_type = dst_type_decide(signal)
            multiplexor(signal, message)

    build_msg_sig(spec.common)
    spec.common.signature = device_struct_sig(spec.common)

    common_c = tpl.tpl["common_c.jinja"].render(spec=spec, date=date())
    common_h = tpl.tpl["common_h.jinja"].render(spec=spec, date=date())

    return common_c, common_h



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'mainwindow.ui'
##
## Created by: Qt User Interface Compiler version 5.14.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import (
    QCoreApplication,
    QMetaObject,
    QObject,
    QPoint,
    QRect,
    QSize,
    QUrl,
    Qt,
)
from PySide2.QtGui import (
    QBrush,
    QColor,
    QConicalGradient,
    QFont,
    QFontDatabase,
    QIcon,
    QLinearGradient,
    QPalette,
    QPainter,
    QPixmap,
    QRadialGradient,
)
from PySide2.QtWidgets import *


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(390, 317)
        self.actionOpen = QAction(MainWindow)
        self.actionOpen.setObjectName(u"actionOpen")
        self.actionSave = QAction(MainWindow)
        self.actionSave.setObjectName(u"actionSave")
        self.actionValidate = QAction(MainWindow)
        self.actionValidate.setObjectName(u"actionValidate")
        self.actionLogs = QAction(MainWindow)
        self.actionLogs.setObjectName(u"actionLogs")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.horizontalLayout = QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 9)
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.scrollArea = QScrollArea(self.centralwidget)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setMaximumSize(QSize(400, 16777215))
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaContents = QWidget()
        self.scrollAreaContents.setObjectName(u"scrollAreaContents")
        self.scrollAreaContents.setGeometry(QRect(0, 0, 336, 195))
        self.verticalLayout = QVBoxLayout(self.scrollAreaContents)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.scrollArea.setWidget(self.scrollAreaContents)

        self.verticalLayout_2.addWidget(self.scrollArea)

        self.addButton = QPushButton(self.centralwidget)
        self.addButton.setObjectName(u"addButton")
        self.addButton.setMaximumSize(QSize(400, 16777215))

        self.verticalLayout_2.addWidget(self.addButton)

        self.logsButton = QPushButton(self.centralwidget)
        self.logsButton.setObjectName(u"logsButton")
        self.logsButton.setMaximumSize(QSize(400, 16777215))
        self.logsButton.setCheckable(True)

        self.verticalLayout_2.addWidget(self.logsButton)

        self.horizontalLayout.addLayout(self.verticalLayout_2)

        self.logDetailsLayout = QVBoxLayout()
        self.logDetailsLayout.setObjectName(u"logDetailsLayout")

        self.horizontalLayout.addLayout(self.logDetailsLayout)

        self.deviceDetailsLayout = QVBoxLayout()
        self.deviceDetailsLayout.setObjectName(u"deviceDetailsLayout")

        self.horizontalLayout.addLayout(self.deviceDetailsLayout)

        self.messageDetailsLayout = QVBoxLayout()
        self.messageDetailsLayout.setObjectName(u"messageDetailsLayout")

        self.horizontalLayout.addLayout(self.messageDetailsLayout)

        self.signalDetailsLayout = QVBoxLayout()
        self.signalDetailsLayout.setObjectName(u"signalDetailsLayout")

        self.horizontalLayout.addLayout(self.signalDetailsLayout)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 390, 20))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionValidate)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)

    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(
            QCoreApplication.translate("MainWindow", u"MainWindow", None)
        )
        self.actionOpen.setText(QCoreApplication.translate("MainWindow", u"Open", None))
        self.actionSave.setText(QCoreApplication.translate("MainWindow", u"Save", None))
        self.actionValidate.setText(
            QCoreApplication.translate("MainWindow", u"Validate", None)
        )
        self.actionLogs.setText(QCoreApplication.translate("MainWindow", u"Logs", None))
        self.addButton.setText(QCoreApplication.translate("MainWindow", u"Add", None))
        self.logsButton.setText(QCoreApplication.translate("MainWindow", u"Logs", None))
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))

    # retranslateUi



# This Python file uses the following encoding: utf-8
import sys
import json

from ww import f

from PySide2.QtWidgets import *

from PySide2.QtGui import QKeySequence
from PySide2.QtCore import Qt

from .spec import *
from .validator import validate

from .gui_lib.mainwindow import Ui_MainWindow
from .gui_lib.devicewidget import Ui_DeviceWidget
from .gui_lib.devicedetails import Ui_DeviceDetails
from .gui_lib.messagewidget import Ui_MessageWidget
from .gui_lib.messagedetails import Ui_MessageDetails
from .gui_lib.signalwidget import Ui_SignalWidget
from .gui_lib.signaldetails import Ui_SignalDetails
from .gui_lib.logwidget import Ui_LogWidget
from .gui_lib.logdetails import Ui_LogDetails
from .gui_lib.cfgwidget import Ui_CfgWidget
from .gui_lib.cfgdetails import Ui_CfgDetails
from .gui_lib.cmdwidget import Ui_CmdWidget
from .gui_lib.cmddetails import Ui_CmdDetails
from .gui_lib.cmdarg import Ui_CmdArg

from datetime import datetime


class FakeParent:
    def __init__(self):
        return

    def setVisible(self, visibility: bool):
        return


class NodeDetails(QWidget):
    """ Node (Device, Message, Signal, Log, Config, Command) interface """

    def __init__(self, gui, node, parent=None, details=None, layout=None):
        QWidget.__init__(self)
        self.node = node
        self.gui = gui
        self.parent = parent
        self.atts = []
        self.details = None
        self.Details = details
        self.layout = layout
        self.children = []
        self.details_button = None

    def load_atts(self, ui, node):
        self.atts = []
        return

    def connect_atts(self):
        def store(att, set_f):
            def closure():
                set_f(att.text())
                self.gui.reload()

            return closure
             
        for att, _, set_f in self.atts:
            att.editingFinished.connect(store(att, set_f))
            #att.textChanged.connect(set_f)
            #att.textChanged.connect(self.gui.reload)

    def save(self):
        for child in self.children:
            child.save()

        for att, get_f, set_f in self.atts:
            set_f(att.text())

        self.gui.reload()

    def delete(self):
        print("delete", self.node)
        r = self.gui.spec.rm_node(self.node)
        self.setVisible(False)
        self.parent.setVisible(False)
        return

    def reload(self):
        for child in self.children:
            child.reload()

        for att, get_f, set_f in self.atts:
            att.setText(str(get_f()))

    def raise_widget(self, checked):
        self.setVisible(checked)

    def open_details(self, checked):
        if checked == False:
            for child in self.children:
                child.open_details(False)
                if not child.details_button is None:
                    child.details_button.setChecked(False)

        if self.details_button is None:
            return

        if self.details is None:
            self.details = self.Details(self.gui, self.node, self)
            self.layout.addWidget(self.details)
            self.children.append(self.details)

        self.details.setVisible(checked)


class SignalDetails(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.nameEdit, node.get_name, node.set_name),
            (ui.startEdit, node.get_start, node.set_start),
            (ui.lengthEdit, node.get_length, node.set_length),
            (ui.muxEdit, node.get_mux, node.set_mux),
            (ui.muxCountEdit, node.get_mux_count, node.set_mux_count),
            (ui.typeEdit, node.get_type, node.set_type),
            (ui.commentEdit, node.get_comment, node.set_comment),
            (ui.minValueEdit, node.get_min_value, node.set_min_value),
            (ui.maxValueEdit, node.get_max_value, node.set_max_value),
            (ui.byteOrderEdit, node.get_byte_order, node.set_byte_order),
            (ui.scaleEdit, node.get_scale, node.set_scale),
            (ui.offsetEdit, node.get_offset, node.set_offset),
        ]

    def __init__(self, gui, node: "Signal", parent):
        NodeDetails.__init__(self, gui, node, parent)

        ui = self.ui = Ui_SignalDetails()
        self.ui.setupUi(self)

        self.load_atts(ui, node)
        self.connect_atts()

        self.ui.signalDeleteButton.clicked.connect(self.delete)

        self.reload()
        self.setVisible(False)

        self.children = []


class SignalWidget(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.name, node.get_name, node.set_name),
            (ui.start, node.get_start, node.set_start),
            (ui.length, node.get_length, node.set_length),
        ]

    def __init__(self, gui, node, details=None, layout=None):
        NodeDetails.__init__(self, gui, node, details=details, layout=layout)

        ui = self.ui = Ui_SignalWidget()
        self.ui.setupUi(self)

        self.load_atts(self.ui, self.node)

        self.details_button = self.ui.signalDetailsButton
        self.details_button.clicked.connect(self.open_details)

        self.reload()


class MessageDetails(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.nameEdit, node.get_name, node.set_name),
            (ui.idEdit, node.get_id, node.set_id),
            (ui.dlcEdit, node.get_dlc, node.set_dlc),
            (ui.frequencyEdit, node.get_frequency, node.set_frequency),
        ]

    def __init__(self, gui, node: "Message", parent):
        NodeDetails.__init__(self, gui, node, parent)

        self.ui = Ui_MessageDetails()
        self.ui.setupUi(self)

        self.load_atts(self.ui, node)
        self.connect_atts()

        for sig in self.node.signals.values():
            self.add_node(sig)

        self.ui.deleteMessageButton.clicked.connect(self.delete)
        self.ui.addButton.clicked.connect(self.add_node)

        self.reload()
        self.setVisible(False)

    def add_node(self, node=None):
        if node == None or node == False:
            node = Signal()

        self.node.signals[node.name] = node
        s = SignalWidget(self.gui, node, SignalDetails, self.gui.ui.signalDetailsLayout)
        self.ui.verticalLayout_2.addWidget(s)
        self.children.append(s)


class MessageWidget(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.id, node.get_id, node.set_id),
            (ui.name, node.get_name, node.set_name),
            (ui.dlc, node.get_dlc, node.set_dlc),
            (ui.frequency, node.get_frequency, node.set_frequency),
        ]

    def __init__(self, gui, node: "Message", details=None, layout=None):
        NodeDetails.__init__(self, gui, node, details=details, layout=layout)
        self.ui = Ui_MessageWidget()
        self.ui.setupUi(self)

        self.load_atts(self.ui, self.node)
        self.reload()

        self.details_button = self.ui.messageDetailsButton
        self.details_button.clicked.connect(self.open_details)


class ArgDetails(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.nameEdit, node.get_name, node.set_name),
            (ui.commentEdit, node.get_comment, node.set_comment),
            (ui.idEdit, node.get_id, node.set_id),
        ]

    def __init__(self, gui, node: "Argument", parent):
        NodeDetails.__init__(self, gui, node, parent)

        self.ui = Ui_CmdArg()
        self.ui.setupUi(self)

        self.load_atts(self.ui, node)
        self.connect_atts()

        self.ui.deleteArgButton.clicked.connect(self.delete)

        self.reload()
        self.setVisible(True)


class CmdDetails(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (self.ui.nameEdit, node.get_name, node.set_name),
            (self.ui.n_argsEdit, node.get_n_args, node.set_n_args),
            (self.ui.commentEdit, node.get_comment, node.set_comment),
            (self.ui.idEdit, node.get_id, node.set_id),
        ]

    def __init__(self, gui, node, parent):
        NodeDetails.__init__(self, gui, node, parent)

        self.ui = Ui_CmdDetails()
        self.ui.setupUi(self)

        self.load_atts(self.ui, node)
        self.connect_atts()

        self.ui.deleteCmdButton.clicked.connect(self.delete)

        self.reload()
        self.setVisible(True)

        self.ui.addArgButton.clicked.connect(self.add_arg)

        for arg in node.args.values():
            self.add_arg(arg)

    def add_arg(self, arg=None):
        if arg == None or arg == False:
            arg = Argument()
        
        print("add argument")
        self.node.args[arg.name] = arg
        arg_widget = ArgDetails(self.gui, arg, FakeParent())
        self.children.append(arg_widget)
        self.ui.CmdArgContents.addWidget(arg_widget)


class CmdWidget(QWidget):
    def __init__(self, gui, parent):
        QWidget.__init__(self)
        self.ui = Ui_CmdWidget()
        self.ui.setupUi(self)

        self.setVisible(False)

        self.parent = parent

        self.gui = gui

        self.details_button = None

        self.children = []

        self.ui.addCmdButton.clicked.connect(self.add_cmd)
        for cmd in self.parent.cmds.values():
            self.add_cmd(cmd)

    def save(self):
        for child in self.children:
            child.save()

        self.gui.reload()

    def reload(self):
        for child in self.children:
            child.reload()

    def add_cmd(self, cmd=None):
        if not cmd:
            cmd = Command()

        self.parent.cmds[cmd.name] = cmd
        cmd_details = CmdDetails(self.gui, cmd, FakeParent())
        self.ui.CmdScrollContents.addWidget(cmd_details)
        self.children.append(cmd_details)

    def open_details(self, checked):
        for child in self.children:
            child.open_details(False)
            if not child.details_button is None:
                child.details_button.setChecked(False)


class CfgDetails(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.nameEdit, node.get_name, node.set_name),
            (ui.idEdit, node.get_id, node.set_id),
            (ui.commentEdit, node.get_comment, node.set_comment),
        ]

    def __init__(self, gui, node, parent):
        NodeDetails.__init__(self, gui, node, parent)

        self.ui = Ui_CfgDetails()
        self.ui.setupUi(self)

        self.load_atts(self.ui, node)
        self.connect_atts()

        self.ui.deleteCfgButton.clicked.connect(self.delete)

        self.reload()
        self.setVisible(True)


class CfgWidget(QWidget):
    def __init__(self, gui, parent):
        QWidget.__init__(self)
        self.ui = Ui_CfgWidget()
        self.ui.setupUi(self)

        self.setVisible(False)

        self.parent = parent

        self.gui = gui

        self.ui.addCfgButton.clicked.connect(self.add_cfg)

        self.children = []

        self.details_button = None

        for cfg in self.parent.cfgs.values():
            self.add_cfg(cfg)

    def open_details(self, checked):
        for child in self.children:
            child.open_details(False)
            if not child.details_button is None:
                child.details_button.setChecked(False)

    def save(self):
        for child in self.children:
            child.save()

        self.gui.reload()

    def reload(self):
        for child in self.children:
            child.reload()

    def add_cfg(self, cfg=None):
        if not cfg:
            cfg = Config()

        self.parent.cfgs[cfg.name] = cfg
        cfg_details = CfgDetails(self.gui, cfg, FakeParent())
        self.ui.CfgScrollContents.addWidget(cfg_details)
        self.children.append(cfg_details)


class DeviceDetails(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.idEdit, node.get_id, node.set_id),
            (ui.nameEdit, node.get_name, node.set_name),
        ]

    def __init__(self, gui, node: "Device", parent):
        NodeDetails.__init__(self, gui, node, parent)

        self.ui = Ui_DeviceDetails()
        self.ui.setupUi(self)

        self.load_atts(self.ui, self.node)
        self.connect_atts()

        self.children = []

        self.cfg_widget = CfgWidget(self.gui, self.node)
        self.children.append(self.cfg_widget)
        self.ui.cfgDetails.addWidget(self.cfg_widget)

        self.cmd_widget = CmdWidget(self.gui, self.node)
        self.children.append(self.cmd_widget)
        self.ui.cmdDetails.addWidget(self.cmd_widget)

        self.ui.addButton.clicked.connect(self.add_node)
        self.ui.cfgsButton.clicked.connect(self.open_cfg)
        self.ui.cmdsButton.clicked.connect(self.open_cmd)

        for msg in self.node.msgs.values():
            self.add_node(msg)

        self.ui.deleteDeviceButton.clicked.connect(self.delete)

        self.reload()

        self.setVisible(False)

    def open_cfg(self, clicked):
        self.cfg_widget.setVisible(clicked)
        return

    def open_cmd(self, clicked):
        self.cmd_widget.setVisible(clicked)
        return

    def add_node(self, node=None):
        if node == None or node == False:
            node = Message()

        self.node.msgs[node.name] = node
        m = MessageWidget(
            self.gui, node, MessageDetails, self.gui.ui.messageDetailsLayout
        )
        self.ui.verticalLayout.addWidget(m)
        self.children.append(m)


class DeviceWidget(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.id, node.get_id, node.set_id),
            (ui.name, node.get_name, node.set_name),
        ]

    def __init__(self, gui, node, details=None, layout=None):
        NodeDetails.__init__(self, gui, node, details=details, layout=layout)

        self.ui = Ui_DeviceWidget()
        self.ui.setupUi(self)

        self.load_atts(self.ui, self.node)
        self.reload()

        self.children = []

        self.details_button = self.ui.deviceDetailsButton
        self.details_button.clicked.connect(self.open_details)


class LogDetails(NodeDetails):
    def load_atts(self, ui, node):
        self.atts = [
            (ui.idEdit, node.get_id, node.set_id),
            (ui.nameEdit, node.get_name, node.set_name),
            (ui.n_argsEdit, node.get_n_args, node.set_n_args),
            (ui.commentEdit, node.get_comment, node.set_comment),
            (ui.stringEdit, node.get_string, node.set_string),
        ]

    def __init__(self, gui, node, parent):
        NodeDetails.__init__(self, gui, node, parent)

        self.ui = Ui_LogDetails()
        self.ui.setupUi(self)

        self.load_atts(self.ui, node)
        self.connect_atts()

        self.ui.logDeleteButton.clicked.connect(self.delete)

        self.reload()
        self.setVisible(True)

        self.children = []


class LogWidget(QWidget):
    def __init__(self, gui, parent):
        self.Window = True
        QWidget.__init__(self)
        self.ui = Ui_LogWidget()
        self.ui.setupUi(self)

        self.parent = parent
        self.gui = gui

        self.atts = []

        self.children = []

        self.reload()

        self.details = None

        self.ui.addLogButton.clicked.connect(self.add_log)
        for log in self.parent.logs.values():
            self.add_log(log)

    def reload(self):
        return

    def save(self):
        for child in self.children:
            child.save()

        self.gui.reload()

    def add_log(self, log=None):
        if log == None or log == False:
            log = Log()

        self.parent.logs[log.name] = log
        log_widget = LogDetails(self.gui, log, FakeParent())
        self.ui.logContents.addWidget(log_widget)
        self.children.append(log_widget)


class Gui(QMainWindow):
    def __init__(self, logger):
        self.logger = logger
        QMainWindow.__init__(self)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.ui.actionOpen.triggered.connect(self.open_json)
        self.ui.actionSave.triggered.connect(self.save_json)
        self.ui.actionValidate.triggered.connect(self.validate)

        self.spec = Spec()
        self.children = []

        shortcutSave = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_S), self)
        shortcutSave.setContext(Qt.ApplicationShortcut)
        shortcutSave.activated.connect(self.save_json)

        shortcutOpen = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_O), self)
        shortcutOpen.setContext(Qt.ApplicationShortcut)
        shortcutOpen.activated.connect(self.open_json)

        self.ui.addButton.clicked.connect(self.add_device)
        self.ui.logsButton.clicked.connect(self.open_log_widget)

        self.log_widget = None

    def open_log_widget(self, checked):
        if self.log_widget is None:
            print("log not opened")
            return

        self.log_widget.setVisible(checked)

    def validate(self):
        msg = QMessageBox(self)
        msg.setStandardButtons(QMessageBox.Ok)
        r, m = validate(self.logger, "{}", spec=self.spec)
        if r:
            msg.setIcon(QMessageBox.Information)
            msg.setText("Spec passed")
        else:
            msg.setIcon(QMessageBox.Warning)
            msg.setText("Failed:" + m)

        msg.show()

        return r, m

    def add_device(self, device=None):
        if type(device) != Device and type(device) != Common:
            device = Device(id=0, name="", msgs={})

            r = self.spec.add_device(device)
            if r == False:
                print("Failed to create device")
                return

        dev_widget = DeviceWidget(
            self, device, details=DeviceDetails, layout=self.ui.deviceDetailsLayout
        )
        self.ui.verticalLayout.addWidget(dev_widget)
        self.children.append(dev_widget)

    def open_json(self):
        filename = QFileDialog.getOpenFileName(
            self, self.tr("Open JSON"), self.tr("JSON (*.json)")
        )
        self.load_json(filename[0])

    def save_json(self):
        for child in self.children:
            child.save()

        r, m = self.validate()
        if r == False:
            return

        filename = QFileDialog.getSaveFileName(
            self, self.tr("Open JSON"), self.tr("JSON (*.json)")
        )
        if filename[0] == "":
            msg = QMessageBox(self)
            msg.setStandardButtons(QMessageBox.Ok)
            msg.setIcon(QMessageBox.Warning)
            msg.setText(f("{filename} is not a valid filename"))
            msg.show()
            return

        with open(filename[0], "w") as f:
            j = self.spec.compile()
            f.write(json.dumps(j, indent=4))

    def load_json(self, filename):
        if filename == "":
            msg = QMessageBox(self)
            msg.setStandardButtons(QMessageBox.Ok)
            msg.setIcon(QMessageBox.Warning)
            msg.setText(f("'{filename}' is not a valid filename"))
            msg.show()
            return

        with open(filename) as f:
            j = json.loads(f.read())

        self.spec.decompile(j)

        for device in sorted(self.spec.devices.values(), key=lambda x: x.id):
            self.add_device(device)

        self.add_device(self.spec.common)

        self.log_widget = LogWidget(self, self.spec)
        self.log_widget.setVisible(False)
        self.ui.logDetailsLayout.addWidget(self.log_widget)

    def reload(self):
        for node in self.children:
            node.reload()

        self.spec.normalize()


def gui(logger, file):
    app = QApplication([])
    window = Gui(logger)
    if file != None and file != "":
        window.load_json(file)
    window.show()
    sys.exit(app.exec_())



import sys
import logging
import json
import subprocess, os

import click

from .dbc_reader import read_dbc, init
from .dbc_writer import write_dbc
from .c_generator import c_gen
from .gui import gui

# from .gui2 import gui2
from .validator import validate
from .spec import Spec
from .docs import generate_docs


def setup_logging() -> logging.Logger:
    """ Setup Logger.

    :return: logger object.
    """

    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger(__name__)

    handler = logging.FileHandler("fcp.log")
    handler.setLevel(logging.DEBUG)

    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


def get_spec(json_file: str, logger: logging.Logger) -> Spec:
    """ Create Spec from json file path.
    :param json_file: path to the json file.
    :param logger: logger.
    :return: Spec.
    """

    spec = Spec()
    with open(json_file) as f:
        j = json.loads(f.read())

    r, msg = validate(logger, j)
    if not r:
        print(msg)
        exit()

    spec.decompile(j)

    return spec


@click.command(name="read-dbc")
@click.argument("dbc")
@click.argument("json_file")
@click.argument("device_config")
def read_dbc_cmd(dbc: str, json_file: str, device_config: str):
    """ Transform a DBC file into FCP json.
    :param dbc: dbc file path.
    :param json_file: FCP json file path.
    :param device_config: json with device name configuration.

    device_config example:

    ```json
    {
	"0": "amk_0",
	"1": "amk_1",
	"2": "amk_2",
	"3": "amk_3",
	"8": "dcu",
	"9": "te",
	"10": "dash",
	"13": "arm",
	"14": "bms_master",
	"15": "bms_verbose",
	"16": "iib",
	"20": "ahrsf",
	"21": "ahrsr",
	"22": "gpsf",
	"23": "gpsr",
	"29": "isa"
    }```
    """
    logger = setup_logging()
    read_dbc(dbc, json_file, device_config, logger)


@click.command(name="write_dbc")
@click.argument("json_file")
@click.argument("dbc")
def write_dbc_cmd(json_file: str, dbc: str):
    """ Transform FCP json file into a DBC
    :param json_file: FCP json file path.
    :param dbc: dbc file path.
    """

    logger = setup_logging()

    with open(json_file) as f:
        j = json.loads(f.read())
    write_dbc(j, dbc, logger)


@click.command(name="c_gen")
@click.argument("templates")
@click.argument("output")
@click.argument("json_file")
@click.argument("skel")
@click.option("--noformat", is_flag=True, default=False)
def c_gen_cmd(templates: str, output: str, json_file: str, skel: str, noformat: bool):
    """ Transform FCP json into a C library.
    :param templates: jinja template directory.
    :param output: output directory.
    :param json_file: FCP json file path.
    :param skel: C skeleton directory.
    """

    logger = setup_logging()
    c_gen(templates, output, json_file, skel, logger)

    if noformat == False:
        subprocess.run(
            "clang-format -i -style=Webkit *.c *.h",
            cwd=os.path.abspath(output),
            shell=True,
        )
        logger.info("clang-format clean up ✅")


@click.command(name="init")
@click.argument("json_file")
def init_cmd(json_file: str):
    """ Create a basic FCP json file. 
    :param json_file: FCP json file path.
    """

    logger = setup_logging()
    init(json_file, logger)


@click.command(name="validate")
@click.argument("json_file")
def validate_cmd(json_file: str):
    """ Verify correctness of FCP json file.
    :param json_file: FCP json file path.
    """

    logger = setup_logging()
    with open(json_file) as f:
        j = f.read()

    r, msg = validate(logger, json.loads(j))
    if r:
        print("JSON validated")
    else:
        print(msg)


@click.command(name="gui")
@click.argument("file", required=False)
def gui_cmd(file: str):
    """ Launch FCP json editor GUI.
    :param file: Optional FCP json file path
    """
    logger = setup_logging()
    gui(logger, file)


@click.command()
@click.argument("json_file")
def dump_dev_list(json_file):
    logger = setup_logging()
    spec = get_spec(json_file, logger)

    for dev in spec.devices.keys():
        print(dev)


@click.command()
@click.argument("json_file")
@click.argument("device", required=False)
def dump_msg_list(json_file, device):
    logger = setup_logging()

    spec = get_spec(json_file, logger)

    for dev_name, dev in spec.devices.items():
        if device != None and device != dev_name:
            continue

        for msg in dev.msgs.keys():
            print(msg)


@click.command()
@click.argument("json_file")
@click.argument("device", required=False)
def dump_cfg_list(json_file, device):
    logger = setup_logging()

    spec = get_spec(json_file, logger)

    for dev_name, dev in spec.devices.items():
        if device != None and device != dev_name:
            continue

        for cfg in dev.cfgs.keys():
            print(cfg)


@click.command()
@click.argument("json_file")
@click.argument("device", required=False)
def dump_cmd_list(json_file, device):
    logger = setup_logging()

    spec = get_spec(json_file, logger)

    for dev_name, dev in spec.devices.items():
        if device != None and device != dev_name:
            continue

        for cmd in dev.cmds.keys():
            print(cmd)


@click.command()
@click.argument("json_file")
def dump_log_list(json_file):
    logger = setup_logging()
    spec = get_spec(json_file, logger)

    for log in spec.logs.keys():
        print(log)


@click.command()
@click.argument("json_file")
@click.argument("message", required=False)
def dump_signal_list(json_file, message):
    logger = setup_logging()
    spec = get_spec(json_file, logger)

    for dev in spec.devices.values():
        for msg in dev.msgs.values():
            if message != None and msg.name != message:
                continue
            for signal in msg.signals.values():
                print(signal)


@click.command()
@click.argument("json_file")
@click.argument("log")
def print_log(json_file, log):
    logger = setup_logging()
    spec = get_spec(json_file, logger)

    print(spec.get_log(log))


@click.command()
@click.argument("json_file")
@click.argument("dev")
def print_dev(json_file, dev):
    logger = setup_logging()

    spec = get_spec(json_file, logger)

    print(spec.get_device(dev))


@click.command()
@click.argument("json_file")
@click.argument("message")
def print_msg(json_file, message):
    logger = setup_logging()
    spec = get_spec(json_file, logger)

    for dev in spec.devices.values():
        for name, msg in dev.msgs.items():
            if name == message:
                print(dev.name)
                print(msg)


@click.command()
@click.argument("json_file")
@click.argument("signal")
def print_signal(json_file, signal):
    logger = setup_logging()

    spec = get_spec(json_file, logger)

    for dev in spec.devices.values():
        for msg in dev.msgs.values():
            for name, sig in msg.signals.items():
                if name == signal:
                    print(dev.name, msg.name)
                    print(sig)


@click.command("docs")
@click.argument("json_file")
@click.argument("out")
@click.argument("link_location", required=False)
def docs(json_file: str, out: str, link_location: str):
    """ Generate FCP documentation. 
    :param json_file: FCP json file path.
    :param out: output directory.
    """

    if link_location == None:
        link_location = "."

    logger = setup_logging()

    spec = get_spec(json_file, logger)
    generate_docs(spec, out, link_location, logger)


@click.group(invoke_without_command=True)
@click.option("--version", is_flag=True, default=False)
def main(version):
    VERSION = "0.1"

    if version:
        click.echo(VERSION)
    pass


main.add_command(read_dbc_cmd)
main.add_command(write_dbc_cmd)
main.add_command(c_gen_cmd)
main.add_command(init_cmd)
main.add_command(validate_cmd)
main.add_command(gui_cmd)
main.add_command(dump_dev_list)
main.add_command(dump_msg_list)
main.add_command(dump_cfg_list)
main.add_command(dump_cmd_list)
main.add_command(dump_log_list)
main.add_command(dump_signal_list)
main.add_command(print_log)
main.add_command(print_dev)
main.add_command(print_msg)
main.add_command(print_signal)
main.add_command(docs)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'messagedetails.ui',
# licensing of 'messagedetails.ui' applies.
#
# Created: Sun Nov  3 00:30:06 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_MessageDetails(object):
    def setupUi(self, MessageDetails):
        MessageDetails.setObjectName("MessageDetails")
        MessageDetails.resize(419, 283)
        self.verticalLayout = QtWidgets.QVBoxLayout(MessageDetails)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(MessageDetails)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.nameLabel = QtWidgets.QLabel(self.groupBox)
        self.nameLabel.setObjectName("nameLabel")
        self.horizontalLayout.addWidget(self.nameLabel)
        self.nameEdit = QtWidgets.QLineEdit(self.groupBox)
        self.nameEdit.setObjectName("nameEdit")
        self.horizontalLayout.addWidget(self.nameEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.idLabel = QtWidgets.QLabel(self.groupBox)
        self.idLabel.setObjectName("idLabel")
        self.horizontalLayout_2.addWidget(self.idLabel)
        self.idEdit = QtWidgets.QLineEdit(self.groupBox)
        self.idEdit.setObjectName("idEdit")
        self.horizontalLayout_2.addWidget(self.idEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.dlcLabel = QtWidgets.QLabel(self.groupBox)
        self.dlcLabel.setObjectName("dlcLabel")
        self.horizontalLayout_3.addWidget(self.dlcLabel)
        self.dlcEdit = QtWidgets.QLineEdit(self.groupBox)
        self.dlcEdit.setObjectName("dlcEdit")
        self.horizontalLayout_3.addWidget(self.dlcEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.frequencyLabel = QtWidgets.QLabel(self.groupBox)
        self.frequencyLabel.setObjectName("frequencyLabel")
        self.horizontalLayout_4.addWidget(self.frequencyLabel)
        self.frequencyEdit = QtWidgets.QLineEdit(self.groupBox)
        self.frequencyEdit.setObjectName("frequencyEdit")
        self.horizontalLayout_4.addWidget(self.frequencyEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.signalContents = QtWidgets.QWidget()
        self.signalContents.setGeometry(QtCore.QRect(0, 0, 375, 66))
        self.signalContents.setObjectName("signalContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.signalContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.scrollArea.setWidget(self.signalContents)
        self.verticalLayout_3.addWidget(self.scrollArea)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.addButton = QtWidgets.QPushButton(self.groupBox)
        self.addButton.setObjectName("addButton")
        self.horizontalLayout_5.addWidget(self.addButton)
        self.deleteMessageButton = QtWidgets.QPushButton(self.groupBox)
        self.deleteMessageButton.setObjectName("deleteMessageButton")
        self.horizontalLayout_5.addWidget(self.deleteMessageButton)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(MessageDetails)
        QtCore.QMetaObject.connectSlotsByName(MessageDetails)

    def retranslateUi(self, MessageDetails):
        MessageDetails.setWindowTitle(
            QtWidgets.QApplication.translate(
                "MessageDetails", "MessageDetails", None, -1
            )
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("MessageDetails", "Message", None, -1)
        )
        self.nameLabel.setText(
            QtWidgets.QApplication.translate("MessageDetails", "name", None, -1)
        )
        self.idLabel.setText(
            QtWidgets.QApplication.translate("MessageDetails", "id", None, -1)
        )
        self.dlcLabel.setText(
            QtWidgets.QApplication.translate("MessageDetails", "dlc", None, -1)
        )
        self.frequencyLabel.setText(
            QtWidgets.QApplication.translate("MessageDetails", "frequency", None, -1)
        )
        self.addButton.setText(
            QtWidgets.QApplication.translate("MessageDetails", "Add", None, -1)
        )
        self.deleteMessageButton.setText(
            QtWidgets.QApplication.translate("MessageDetails", "Delete", None, -1)
        )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'messagewidget.ui',
# licensing of 'messagewidget.ui' applies.
#
# Created: Sun Nov  3 00:30:06 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_MessageWidget(object):
    def setupUi(self, MessageWidget):
        MessageWidget.setObjectName("MessageWidget")
        MessageWidget.resize(268, 41)
        self.horizontalLayout = QtWidgets.QHBoxLayout(MessageWidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.name = QtWidgets.QLabel(MessageWidget)
        self.name.setText("")
        self.name.setObjectName("name")
        self.horizontalLayout.addWidget(self.name)
        self.id = QtWidgets.QLabel(MessageWidget)
        self.id.setText("")
        self.id.setObjectName("id")
        self.horizontalLayout.addWidget(self.id)
        self.dlc = QtWidgets.QLabel(MessageWidget)
        self.dlc.setText("")
        self.dlc.setObjectName("dlc")
        self.horizontalLayout.addWidget(self.dlc)
        self.frequency = QtWidgets.QLabel(MessageWidget)
        self.frequency.setText("")
        self.frequency.setObjectName("frequency")
        self.horizontalLayout.addWidget(self.frequency)
        self.messageDetailsButton = QtWidgets.QPushButton(MessageWidget)
        self.messageDetailsButton.setCheckable(True)
        self.messageDetailsButton.setObjectName("messageDetailsButton")
        self.horizontalLayout.addWidget(self.messageDetailsButton)

        self.retranslateUi(MessageWidget)
        QtCore.QMetaObject.connectSlotsByName(MessageWidget)

    def retranslateUi(self, MessageWidget):
        MessageWidget.setWindowTitle(
            QtWidgets.QApplication.translate("MessageWidget", "MessageWidget", None, -1)
        )
        self.messageDetailsButton.setText(
            QtWidgets.QApplication.translate("MessageWidget", "Details", None, -1)
        )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'cmdwidget.ui',
# licensing of 'cmdwidget.ui' applies.
#
# Created: Sun Nov  3 00:30:05 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_CmdWidget(object):
    def setupUi(self, CmdWidget):
        CmdWidget.setObjectName("CmdWidget")
        CmdWidget.resize(400, 300)
        CmdWidget.setMinimumSize(QtCore.QSize(350, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(CmdWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(CmdWidget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self._CmdScrollContents = QtWidgets.QWidget()
        self._CmdScrollContents.setGeometry(QtCore.QRect(0, 0, 356, 209))
        self._CmdScrollContents.setMinimumSize(QtCore.QSize(300, 0))
        self._CmdScrollContents.setObjectName("_CmdScrollContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self._CmdScrollContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.CmdScrollContents = QtWidgets.QVBoxLayout()
        self.CmdScrollContents.setObjectName("CmdScrollContents")
        self.verticalLayout_2.addLayout(self.CmdScrollContents)
        self.scrollArea.setWidget(self._CmdScrollContents)
        self.verticalLayout_4.addWidget(self.scrollArea)
        self.addCmdButton = QtWidgets.QPushButton(self.groupBox)
        self.addCmdButton.setObjectName("addCmdButton")
        self.verticalLayout_4.addWidget(self.addCmdButton)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(CmdWidget)
        QtCore.QMetaObject.connectSlotsByName(CmdWidget)

    def retranslateUi(self, CmdWidget):
        CmdWidget.setWindowTitle(
            QtWidgets.QApplication.translate("CmdWidget", "CmdWidget", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("CmdWidget", "Commands", None, -1)
        )
        self.addCmdButton.setText(
            QtWidgets.QApplication.translate("CmdWidget", "Add Cmd", None, -1)
        )



import sys
import json
from spec import Spec, Device

from input import to_type


def main(args):
    with open(args[1]) as f:
        j = json.loads(f.read())

    spec = Spec(j)

    d = Device()

    name = input("Device name: ")
    d.name = to_type(name, str)
    id = input("Device id: ")
    d.id = to_type(id, int)
    spec.add_device(d.compile())

    with open(args[1], "w") as f:
        f.write(json.dumps(spec.json, sort_keys=True, indent=4))


if __name__ == "__main__":
    main(sys.argv)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'signaldevice.ui',
# licensing of 'signaldevice.ui' applies.
#
# Created: Mon Oct 14 22:57:02 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!



from enum import Enum

# from dataclasses import dataclass, field
from typing import *

import copy
from pprint import pprint

import datetime

from ww import f


def field(default_factory):
    return default_factory()


def make_dict():
    return {}


def normalize(xs: Dict[str, Any], key: Callable[[Any], str] = None):
    """ Update xs dictionary keys according to key. 
        By default key is `lambda x : x.name`

    :param xs: Dictionary containing spec node
    :param key: Function that returns the key for a particular node
    """

    if key == None:
        key = lambda x: x.name

    aux = []

    for k, x in xs.items():
        if k != key(x):
            aux.append((k, key(x)))

    for k, key in aux:
        xs[key] = xs[k]
        del xs[k]


class Log:
    """ Log protocol node.

    :param id: Log integer identifier.
    :param name: Name of the Log node.
    :param n_args: Number of arguments in the Log node.
    :param comment: Description of the Log node
    :param string: Display string for the Log node.
    """

    def __init__(
        self,
        id: int = -1,
        name: str = "",
        n_args: int = 3,
        comment: str = "",
        string: str = "",
    ):
        self.id = id
        self.name = name
        self.n_args = n_args
        self.comment = comment
        self.string = string

        self.creation_date = datetime.datetime.now()

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        d = copy.deepcopy(self.__dict__)
        del d["creation_date"]
        return d

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        self.__dict__.update(d)

    def get_id(self) -> int:
        return self.id

    def get_name(self) -> str:
        return self.name

    def get_n_args(self) -> int:
        return self.n_args

    def get_comment(self) -> str:
        return self.comment

    def get_string(self) -> str:
        return self.string

    def set_id(self, id: int) -> None:
        try:
            self.id = int(id)
        except Exception as e:
            return

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_n_args(self, n_args: int) -> None:
        try:
            self.n_args = int(n_args)
        except Exception as e:
            return

    def set_comment(self, comment: str) -> None:
        try:
            self.comment = comment
        except Exception as e:
            return

    def set_string(self, string: str) -> None:
        try:
            self.string = string
        except Exception as e:
            return

    def __hash__(self):
        return hash((self.name, self.id, self.creation_date))

    def __repr__(self):
        return f(
            "name: {self.name}, id: {self.id}, string: {self.string}, n_args: {self.n_args}, comment: {self.comment}"
        )


class Spec:
    """ FCP root node. Holds all Devices, Messages, Signals, Logs, Configs,
    Commands and Arguments.
    """

    def __init__(self) -> None:
        self.devices = {}
        self.common = Common()
        self.logs = {}
        self.version = "0.2"

    def add_device(self, device: "Device") -> bool:
        """ Add a Device to Spec. 

        :param device: Device to be added
        :return: Operation success status: True - Success, False - Failure
        """
        if device is None:
            return False

        if device.name in self.devices.keys():
            return False

        self.devices[device.name] = device

        return True

    def add_log(self, log: "Log") -> bool:
        """ Add a Log to Spec. 

        :param log: Log to be added
        :return: Operation success status: True - Success, False - Failure
        """

        if log == None:
            return False

        if log.name in self.logs.keys():
            return False

        self.logs[log.name] = log

        return True

    def get_device(self, name: str) -> Optional["Device"]:
        """ Get a Device from Spec by its name.

        :param name: Device name.
        :return: Device or None if not found.
        """
        return self.devices.get(name)

    def get_log(self, name: str) -> Optional["Log"]:
        """ Get a Log from Spec by its name.

        :param name: Log name.
        :return: Log or None if not found.
        """
        return self.logs.get(name)

    def rm_node(self, node: Any) -> None:
        """ Remove a node from Spec.

        :param node: node to be removed.
        """
        if type(node) == Device:
            self.rm_device(node)
        elif type(node) == Message:
            self.rm_message(node)
        elif type(node) == Signal:
            self.rm_signal(node)
        elif type(node) == Config:
            self.rm_config(node)
        elif type(node) == Command:
            self.rm_cmd(node)
        elif type(node) == Log:
            self.rm_log(node)

    def rm_device(self, device: "Device") -> None:
        """ Remove a Device from Spec.

        :param device: Device to be removed.
        """
        devs = [dev.name for dev in self.devices.values() if dev == device]

        for name in devs:
            del self.devices[name]

    def rm_message(self, message: "Message") -> None:
        """ Remove a Message from Spec.

        :param message: Message to be removed.
        """
        for dev in self.devices.values():
            msgs = []
            for msg in dev.msgs.values():
                if msg == message:
                    msgs.append(msg.name)

            for name in msgs:
                dev.rm_msg(name)

    def rm_signal(self, signal: "Signal") -> None:
        """ Remove a Signal from Spec.

        :param signal: Signal to be removed.
        """


        for dev in self.devices.values():
            for msg in dev.msgs.values():
                signals = []
                for sig in msg.signals.values():
                    if sig is signal:
                        print("found signal")
                        signals.append(signal.name)

                for name in signals:
                    msg.rm_signal(name)

    def rm_config(self, config: "Config") -> None:
        """ Remove a Config from Spec.

        :param config: Config to be removed.
        """


        for dev in self.devices.values():
            cfgs = []
            for cfg in dev.cfgs.values():
                if cfg is config:
                    cfgs.append(cfg.name)

            for name in cfgs:
                dev.rm_cfg(name)

    def rm_cmd(self, command: "Command") -> None:
        """ Remove a Command from Spec.

        :param command: Command to be removed.
        """


        for dev in self.devices.values():
            cmds = []
            for cmd in dev.cfgs.values():
                if cmd is command:
                    cmds.append(cmd)

            for name in cmds:
                dev.rm_cmd(name)

    def rm_log(self, log):
        """ Remove a Log from Spec.

        :param log: Log to be removed.
        """
        logs = []

        for l in self.logs.values():
            if log is l:
                logs.append(l.name)

        for name in logs:
            del self.logs[name]

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        d = {"devices": {}, "logs": {}}

        for dev_k, dev_v in self.devices.items():
            d["devices"][dev_k] = dev_v.compile()

        for log_k, log_v in self.logs.items():
            d["logs"][log_k] = log_v.compile()

        d["common"] = self.common.compile()

        d["version"] = self.version

        return d

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        d = copy.deepcopy(d)
        self.devices = {}
        self.logs = {}
        self.common.decompile(d["common"])

        for k, v in d["devices"].items():
            dev = Device()
            dev.decompile(v)
            self.devices[k] = dev

        for k, v in d["logs"].items():
            log = Log()
            log.decompile(v)
            self.logs[k] = log

        self.versionn = d["version"]

    def normalize(self):
        """ Update devices and logs dictionary keys.  """

        normalize(self.devices)
        normalize(self.logs)

        for key, dev in self.devices.items():
            dev.normalize()

    def __repr__(self):
        out = "Spec: \n"
        for device in self.devices.values():
            out += ""
            out += str(device)
            out += "\n"

        return out


class Signal:
    """ Signal node. Represents a CAN signal, similar to a DBC signal.
        
        :param name: Name of the Signal.
        :param start: Start bit
        :param length: Signal bit size.
        :param scale: Scaling applied to the signal's data.
        :param offset: Offset applied to the signal's data.
        :param unit: Unit of the Signal after applying scaling and offset.
        :param comment: Description of the Signal.
        :param min_value: Minimum value allowed to the Signal's data.
        :param max_value: Maximum value allowed to the Signal's data.
        :param type: Type of the Signal's data.
        :param mux: Name of the mux Signal. None if the Signal doesn't belong
        to a multiplexed Message.
        :param mux_count: Number of signals that the mux can reference for this Muxed signal.

    """

    def __init__(
        self,
        name: str = "default_name",
        start: int = 0,
        length: int = 0,
        scale: float = 1,
        offset: float = 0,
        unit: str = "",
        comment: str = "",
        min_value: int = 0,
        max_value: int = 0,
        type: str = "unsigned",
        byte_order: str = "little_endian",
        mux: str = "",
        mux_count: int = 0,
    ):

        self.name = name
        self.start = start
        self.length = length
        self.scale = scale
        self.offset = offset
        self.unit = unit
        self.comment = comment
        self.min_value = min_value
        self.max_value = max_value
        self.type = type
        self.byte_order = byte_order
        self.mux = mux
        self.mux_count = mux_count

        self.creation_date = datetime.datetime.now()

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_start(self, start: int) -> None:
        try:
            self.start = int(start)
        except Exception as e:
            return

    def set_length(self, length: int) -> None:
        try:
            self.length = int(length)
        except Exception as e:
            return

    def set_scale(self, scale: float) -> None:
        try:
            self.scale = float(scale)
        except Exception as e:
            return

    def set_offset(self, offset: float) -> None:
        try:
            self.offset = float(offset)
        except Exception as e:
            return

    def set_unit(self, unit: str) -> None:
        try:
            self.unit = unit
        except Exception as e:
            return

    def set_comment(self, comment: str) -> None:
        try:
            self.comment = comment
        except Exception as e:
            return

    def set_min_value(self, min_value: float) -> None:
        try:
            self.min_value = float(min_value)
        except Exception as e:
            return

    def set_max_value(self, max_value: float) -> None:
        try:
            self.max_value = float(max_value)
        except Exception as e:
            return

    def set_type(self, type: str) -> None:
        try:
            self.type = type
        except Exception as e:
            return

    def set_byte_order(self, byte_order: str) -> None:
        try:
            self.byte_order = byte_order
        except Exception as e:
            return

    def set_mux(self, mux: str) -> None:
        try:
            self.mux = mux
        except Exception as e:
            return

    def set_mux_count(self, mux_count: int) -> None:
        try:
            self.mux_count = int(mux_count)
        except Exception as e:
            return

    def get_name(self) -> str:
        return self.name

    def get_start(self) -> int:
        return self.start

    def get_length(self) -> int:
        return self.length

    def get_scale(self) -> float:
        return self.scale

    def get_offset(self) -> float:
        return self.offset

    def get_unit(self) -> str:
        return self.unit

    def get_comment(self) -> str:
        return self.comment

    def get_min_value(self) -> float:
        return self.min_value

    def get_max_value(self) -> float:
        return self.max_value

    def get_type(self) -> str:
        return self.type

    def get_byte_order(self) -> str:
        return self.byte_order

    def get_mux(self) -> str:
        return self.mux

    def get_mux_count(self) -> int:
        return self.mux_count

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        d = copy.deepcopy(self.__dict__)
        del d["creation_date"]
        return d

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        self.__dict__.update(d)

    def __hash__(self):
        return hash((self.name, self.start, self.length, self.creation_date))

    def __repr__(self):
        return (
            "{"
            + f("""
    name: {self.name},
    start: {self.start}, 
    length: {self.length}, 
    scale: {self.scale}, 
    offset: {self.offset}, 
    unit: {self.unit}, 
    comment: {self.comment}, 
    min: {self.min_value}, 
    max: {self.max_value}, 
    type: {self.type}, 
    byte_order: {self.byte_order}, 
    mux: {self.mux}, 
    mux_count: {self.mux_count}
""")
            + "}"
        )


class Message:
    """ Message node, Represents a CAN message, similar to a DBC message.
        
        :param name: Name of the Message.
        :param id: FST Message identifier, highest 6 bits of the identifier.
        :param dlc: Message DLC.
        :param signals: Dictionary containing the Message signals.
        :param frequency: Transmission period in millisecond. If 0 message
        isn't automatically sent. 
    """

    def __init__(
        self,
        name: str = "default_name",
        id: int = 0,
        dlc: int = 8,
        signals: Dict[str, Signal] = None,
        frequency: int = 0,
    ):

        self.name = name
        self.id = id
        self.dlc = dlc
        self.signals = {} if signals == None else signals
        self.frequency = frequency

        self.creation_date = datetime.datetime.now()

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_id(self, id: int) -> None:
        try:
            self.id = int(id)
        except Exception as e:
            return

    def set_dlc(self, dlc: int) -> None:
        try:
            self.dlc = int(dlc)
        except Exception as e:
            return

    def set_frequency(self, frequency: int) -> None:
        try:
            self.frequency = int(frequency)
        except Exception as e:
            return

    def get_name(self) -> str:
        return self.name

    def get_id(self) -> int:
        return self.id

    def get_dlc(self) -> int:
        return self.dlc

    def get_frequency(self) -> int:
        return self.frequency

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        msgs = {}
        for k, v in self.signals.items():
            msgs[k] = v.compile()

        d = copy.deepcopy(self.__dict__)
        d["signals"] = msgs
        del d["creation_date"]
        return d

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        signals = d["signals"]
        del d["signals"]

        self.__dict__.update(d)
        for key, value in signals.items():
            sig = Signal()
            sig.decompile(value)
            self.signals[key] = sig

    def add_signal(self, signal: Signal) -> bool:
        """ Add a Signal to Message. 

        :param signal: Signal to be added
        :return: Operation success status: True - Success, False - Failure
        """

        if signal == None:
            return False

        if signal.name in self.signals.keys():
            self.signals[signal.name].mux_count += 1
            return True

        self.signals[signal.name] = signal

        return True

    def get_signal(self, name: str) -> Optional[Signal]:
        """ Get a Signal from Message by its name.

        :param name: Signal name.
        :return: Signal or None if not found.
        """

        return self.signals.get(name)

    def rm_signal(self, name: str) -> bool:
        """ Remove a Signal from Spec.

        :param signal: Signal to be removed.
        """
        if self.get_signal(name) is None:
            print("Not found", name)
            return False
        
        print("deleting signal") 
        del self.signals[name]
        return True

    def normalize(self):
        """ Update signals dictionary keys."""

        normalize(self.signals)

    def __hash__(self):
        return hash((self.name, self.id, self.creation_date))

    def __repr__(self):
        return (
            "{"
            + f("""
    name: {self.name}, 
    id: {self.id}, 
    dlc: {self.dlc}, 
    frequency: {self.frequency}
""")
            + "}"
        )


class Argument:
    """ Argument node. Represents a Command Argument.

    :param name: Name of the Argument.
    :param id: Argument identifier.
    :param comment: description of the Argument.
    """

    def __init__(self, name: str = "", id: int = 0, comment: str = ""):

        self.name = name
        self.id = id
        self.comment = comment

        self.creation_date = datetime.datetime.now()

    def get_name(self) -> str:
        return self.name

    def get_comment(self) -> str:
        return self.comment

    def get_id(self) -> int:
        return self.id

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_comment(self, comment: str) -> None:
        try:
            self.comment = comment
        except Exception as e:
            return

    def set_id(self, id: int) -> None:
        try:
            self.id = id
        except Exception as e:
            return

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """
        print("compiling argument")
        d = copy.deepcopy(self.__dict__)
        del d["creation_date"]
        return d

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        self.__dict__.update(d)


class Command:
    """ Command node. Represents a Command.

    :param name: Name of the Command.
    :param n_args: Number of arguments in the Command.
    :param comment: description of the Command.
    :param id: Command identifier.
    :param args: Dictionary containing the Command's input Arguments.
    :param rets: Dictionary containing the Command's output Arguments.
    """

    def __init__(
        self,
        name: str = "",
        n_args: int = 3,
        comment: str = "",
        id: int = 0,
        args: Dict[str, Argument] = None,
        rets: Dict[str, Argument] = None,
    ):

        self.name = name
        self.n_args = n_args
        self.comment = comment
        self.id = id
        self.args = {} if args == None else args
        self.rets = {} if rets == None else rets

        self.creation_date = datetime.datetime.now()

    def get_name(self) -> str:
        return self.name

    def get_n_args(self) -> int:
        return self.n_args

    def get_comment(self) -> str:
        return self.comment

    def get_id(self) -> int:
        return self.id

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_n_args(self, n_args: int) -> None:
        try:
            self.n_args = n_args
        except Exception as e:
            return

    def set_comment(self, comment: str) -> None:
        try:
            self.comment = comment
        except Exception as e:
            return

    def set_id(self, id: int) -> None:
        try:
            self.id = id
        except Exception as e:
            return

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        args = {}
        rets = {}
        print(self.args)
        for k, v in self.args.items():
            args[k] = v.compile()

        for k, v in self.rets.items():
            rets[k] = v.compile()

        att = copy.deepcopy(self.__dict__)
        att["args"] = args
        att["rets"] = rets
        del att["creation_date"]

        return att

    def add_arg(self, arg: Argument) -> None:
        """ Add a input Argument to Command. 

        :param arg: Argument to be added
        :return: Operation success status: True - Success, False - Failure
        """
        self.args[arg.name] = arg

    def add_ret(self, ret: Argument) -> None:
        """ Add a output Argument to Command. 

        :param ret: Argument to be added
        :return: Operation success status: True - Success, False - Failure
        """
        self.rets[ret.name] = ret

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        args = d["args"]
        rets = d["rets"]
        del d["args"]
        del d["rets"]

        self.__dict__.update(d)

        for arg_k, arg_v in args.items():
            arg = Argument()
            arg.decompile(arg_v)
            self.args[arg_k] = arg

        for ret_k, ret_v in rets.items():
            ret = Argument()
            ret.decompile(ret_v)
            self.rets[ret_k] = ret

    def normalize(self):
        return

    def __hash__(self):
        return hash((self.name, self.id, self.creation_date))


class Config:
    """ Config node. Represents a Config.

    :param name: Name of the Config.
    :param id: Config identifier.
    :param comment: description of the Config.
    """

    def __init__(self, name: str = "", id: int = 0, comment: str = ""):

        self.name = name
        self.id = id
        self.comment = comment

        self.creation_date = datetime.datetime.now()

    def get_name(self) -> str:
        return self.name

    def get_id(self) -> int:
        return self.id

    def get_comment(self) -> str:
        return self.comment

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_id(self, id: int) -> None:
        try:
            self.id = id
        except Exception as e:
            return

    def set_comment(self, comment: str) -> None:
        try:
            self.comment = comment
        except Exception as e:
            return

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        d = copy.deepcopy(self.__dict__)
        del d["creation_date"]
        return d

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        self.__dict__.update(d)

    def normalize(self):
        return

    def __hash__(self):
        return hash((self.name, self.id, self.creation_date))


class Device:
    """ Device node, Represents a CAN device.
        
        :param name: Name of the Device.
        :param id: FST Device identifier, lowest 5 bits of the identifier.
        :param msgs: Dictionary containing the Device messages.
        :param cmds: Dictionary containing the Device commands.
        :param cfgs: Dictionary containing the Device configs.
        isn't automatically sent. 
    """

    def __init__(
        self,
        name: str = "default_name",
        id: int = 0,
        msgs: Dict[str, Message] = None,
        cmds: Dict[str, Command] = None,
        cfgs: Dict[str, Config] = None,
    ):

        self.name = name
        self.id = id
        self.msgs = {} if msgs == None else msgs
        self.cmds = {} if cmds == None else cmds
        self.cfgs = {} if cfgs == None else cfgs

        self.creation_date = datetime.datetime.now()

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_id(self, id: int) -> None:
        try:
            self.id = int(id)
        except Exception as e:
            return

    def get_name(self) -> str:
        return self.name

    def get_id(self) -> int:
        return self.id

    def add_cmd(self, cmd: Command) -> None:
        self.cmds[cmd.name] = cmd

    def add_cfg(self, cfg: Config) -> None:
        self.cfgs[cfg.name] = cfg

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        msgs = {}
        cmds = {}
        cfgs = {}

        for msg_k, msg_v in self.msgs.items():
            msgs[msg_k] = msg_v.compile()

        for cmd_k, cmd_v in self.cmds.items():
            cmds[cmd_k] = cmd_v.compile()

        for cfg_k, cfg_v in self.cfgs.items():
            cfgs[cfg_k] = cfg_v.compile()

        att = copy.deepcopy(self.__dict__)
        att["msgs"] = msgs
        att["cmds"] = cmds
        att["cfgs"] = cfgs
        del att["creation_date"]

        return att

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        msgs = d["msgs"]
        cmds = d["cmds"]
        cfgs = d["cfgs"]

        del d["msgs"]
        del d["cmds"]
        del d["cfgs"]

        self.__dict__.update(d)

        for k, v in msgs.items():
            msg = Message()
            msg.decompile(v)
            self.msgs[k] = msg

        for k, v in cmds.items():
            cmd = Command()
            cmd.decompile(v)
            self.cmds[k] = cmd

        for k, v in cfgs.items():
            cfg = Config()
            cfg.decompile(v)
            self.cfgs[k] = cfg

    def add_msg(self, msg: Message) -> bool:
        """ Add a Message to Device. 

        :param msg: Message to be added.
        :return: Operation success status: True - Success, False - Failure
        """
        if msg == None:
            return False

        if msg.name in self.msgs.keys():
            return False

        self.msgs[msg.name] = msg

        return True

    def get_msg(self, name: str) -> Optional[Message]:
        """ Get a Message from Device by its name.

        :param name: Message name.
        :return: Message or None if not found.
        """
        return self.msgs.get(name)

    def rm_msg(self, name: str) -> bool:
        """ Remove a Message from Device.

        :param name: Name of the Message to be removed.
        """
        if self.get_msg(name) is None:
            return False

        del self.msgs[name]
        return True

    def get_cmd(self, name: str) -> Optional[Command]:
        """ Get a Command from Device by its name.

        :param name: Command name.
        :return: Command or None if not found.
        """
        return self.cmds.get(name)

    def rm_cmd(self, name: str) -> bool:
        """ Remove a Command from Device.

        :param name: Name of the Command to be removed.
        """
        if self.get_cmd(name) is None:
            return False

        del self.cmds[name]
        return True

    def get_cfg(self, name: str) -> Optional[Config]:
        """ Get a Config from Device by its name.

        :param name: Config name.
        :return: Config or None if not found.
        """
        return self.cfgs.get(name)

    def rm_cfg(self, name: str) -> bool:
        """ Remove a Config from Device.

        :param name: Name of the Config to be removed.
        """
        if self.get_cfg(name) is None:
            return False

        del self.cfgs[name]
        return True

    def normalize(self):
        """ Update messages, commands and configs dictionary keys.  """
        normalize(self.msgs)
        normalize(self.cmds)
        normalize(self.cfgs)

        for key, msg in self.msgs.items():
            msg.normalize()

        for key, cmd in self.cmds.items():
            cmd.normalize()

        for key, cfg in self.cfgs.items():
            cfg.normalize()

    def __hash__(self):
        return hash((self.name, self.id, self.creation_date))

    def __repr__(self):
        return (
            "{"
            + f("""
    name: {self.name}, 
    id: {self.id}
""")
            + "}"
        )


class Common:
    def __init__(
        self,
        name: str = "common",
        id: int = 0,
        msgs: Dict[str, Message] = None,
        cfgs: Dict[str, Config] = None,
        cmds: Dict[str, Command] = None,
    ):

        self.name = name
        self.id = id
        self.msgs = {} if msgs == None else msgs
        self.cfgs = {} if cfgs == None else cfgs
        self.cmds = {} if cmds == None else cmds

        self.creation_date = datetime.datetime.now()

    def set_name(self, name: str) -> None:
        try:
            self.name = name
        except Exception as e:
            return

    def set_id(self, id: int) -> None:
        try:
            self.id = int(id)
        except Exception as e:
            return

    def get_name(self) -> str:
        return self.name

    def get_id(self) -> int:
        return self.id

    def add_msg(self, msg: Message) -> bool:
        if msg == None:
            return False

        if msg.name in self.msgs.keys():
            return False

        self.msgs[msg.name] = msg
        return True

    def compile(self) -> Dict[str, Any]:
        """ Transform python class node to its dictionary representation.

        :return: A dictionary containing the node parameters
        """

        msgs = {}
        for msg_k, msg_v in self.msgs.items():
            msgs[msg_k] = msg_v.compile()

        att = copy.deepcopy(self.__dict__)
        att["msgs"] = msgs
        del att["creation_date"]
        return att

    def decompile(self, d: Dict[str, Any]) -> None:
        """ Transform node dictionary representation into a python class.

        :param d: Node dictionary
        """
        msgs = d["msgs"]
        del d["msgs"]

        self.__dict__.update(d)

        for key, value in msgs.items():
            msg = Message()
            msg.decompile(value)
            self.msgs[key] = msg

    def __hash__(self):
        return hash((self.name, self.id, self.creation_date))


def make_sid(dev_id: int, msg_id: int) -> int:
    """ Find the sid from the dev_id and the msg_id """
    return msg_id * 32 + dev_id


def decompose_id(sid: int) -> Tuple[int, int]:
    """ Find the dev_id and the msg_id from the sid."""
    return sid & 0x1F, (sid >> 5) & 0x3F



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'cmdarg.ui',
# licensing of 'cmdarg.ui' applies.
#
# Created: Sun Nov  3 00:30:04 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_CmdArg(object):
    def setupUi(self, CmdArg):
        CmdArg.setObjectName("CmdArg")
        CmdArg.resize(400, 300)
        CmdArg.setMinimumSize(QtCore.QSize(300, 0))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(CmdArg)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(CmdArg)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.idLabel = QtWidgets.QLabel(self.groupBox)
        self.idLabel.setObjectName("idLabel")
        self.horizontalLayout_2.addWidget(self.idLabel)
        self.idEdit = QtWidgets.QLineEdit(self.groupBox)
        self.idEdit.setObjectName("idEdit")
        self.horizontalLayout_2.addWidget(self.idEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.nameLabel = QtWidgets.QLabel(self.groupBox)
        self.nameLabel.setObjectName("nameLabel")
        self.horizontalLayout.addWidget(self.nameLabel)
        self.nameEdit = QtWidgets.QLineEdit(self.groupBox)
        self.nameEdit.setObjectName("nameEdit")
        self.horizontalLayout.addWidget(self.nameEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.commentLabel = QtWidgets.QLabel(self.groupBox)
        self.commentLabel.setObjectName("commentLabel")
        self.horizontalLayout_3.addWidget(self.commentLabel)
        self.commentEdit = QtWidgets.QLineEdit(self.groupBox)
        self.commentEdit.setObjectName("commentEdit")
        self.horizontalLayout_3.addWidget(self.commentEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        self.deleteArgButton = QtWidgets.QPushButton(self.groupBox)
        self.deleteArgButton.setObjectName("deleteArgButton")
        self.verticalLayout_3.addWidget(self.deleteArgButton)
        self.verticalLayout_2.addWidget(self.groupBox)

        self.retranslateUi(CmdArg)
        QtCore.QMetaObject.connectSlotsByName(CmdArg)

    def retranslateUi(self, CmdArg):
        CmdArg.setWindowTitle(
            QtWidgets.QApplication.translate("CmdArg", "CmdArg", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("CmdArg", "Argument", None, -1)
        )
        self.idLabel.setText(QtWidgets.QApplication.translate("CmdArg", "id", None, -1))
        self.nameLabel.setText(
            QtWidgets.QApplication.translate("CmdArg", "name", None, -1)
        )
        self.commentLabel.setText(
            QtWidgets.QApplication.translate("CmdArg", "comment", None, -1)
        )
        self.deleteArgButton.setText(
            QtWidgets.QApplication.translate("CmdArg", "Delete", None, -1)
        )



#!/usr/bin/env python3

"""dbc-reader.

Usage:
  dbc-reader read <dbc> <json> [<device-config>]
  dbc-reader (-h | --help)
  dbc-reader --version

Options:
  -h --help     Show this screen.
  --version     Show version.
"""

import logging
import json
import cantools

from .spec import (
    Spec,
    Device,
    Message,
    Signal,
    Log,
    Command,
    Argument,
    Config,
    make_sid,
    decompose_id,
)

from pprint import pprint


def find_type(signal):
    if signal.is_float:
        return "float"
    elif signal.is_signed:
        return "signed"
    else:
        return "unsigned"


def query_user_ids(dev_id):
    if dev_id not in query_user_ids.ids.keys():
        print(dev_id)
        query_user_ids.ids[dev_id] = input("Device name: ")
        return query_user_ids.ids[dev_id]
    else:
        return query_user_ids.ids[dev_id]


query_user_ids.ids = {}


def default_device(device):
    version = Config("version", 0, "git version")
    device.add_cfg(version)


def default_common_log_msg(spec):
    log_msg = Message(name="log", id=0)
    level = Signal(name="level", start=0, length=3)
    n_args = Signal(start=3, length=2, name="n_args")
    err_code = Signal(start=8, length=8, name="err_code")
    arg1 = Signal(start=16, length=16, name="arg1")
    arg2 = Signal(start=32, length=16, name="arg2")
    arg3 = Signal(start=48, length=16, name="arg3")
    log_msg.add_signal(level)
    log_msg.add_signal(n_args)
    log_msg.add_signal(err_code)
    log_msg.add_signal(arg1)
    log_msg.add_signal(arg2)
    log_msg.add_signal(arg3)

    spec.common.add_msg(log_msg)


def default_common_cfg_msg(spec):
    req_get = Message(name="req_get", id=3)
    dst = Signal(name="dst", start=0, length=5)
    id = Signal(name="id", start=8, length=8)

    req_get.add_signal(id)
    req_get.add_signal(dst)
    spec.common.add_msg(req_get)

    ans_get = Message(name="ans_get", id=4)
    id = Signal(name="id", start=0, length=5)
    dst = Signal(name="dst", start=8, length=8)
    data = Signal(name="data", start=16, length=32)

    ans_get.add_signal(id)
    ans_get.add_signal(dst)
    ans_get.add_signal(data)
    spec.common.add_msg(ans_get)

    req_set = Message(name="req_set", id=5)
    dst = Signal(name="dst", start=0, length=5)
    id = Signal(name="id", start=8, length=8)
    data = Signal(name="data", start=16, length=32)

    req_set.add_signal(id)
    req_set.add_signal(dst)
    req_set.add_signal(data)
    spec.common.add_msg(req_set)

    ans_set = Message(name="ans_set", id=6)
    id = Signal(name="id", start=0, length=5)
    dst = Signal(name="dst", start=8, length=8)
    data = Signal(name="data", start=16, length=32)

    ans_set.add_signal(id)
    ans_set.add_signal(dst)
    ans_set.add_signal(data)
    spec.common.add_msg(ans_set)


def default_common_cmd_msg(spec):
    cmd_args_msg = Message(name="send_cmd", id=1)
    dst = Signal(name="dst", start=0, length=5)
    id = Signal(name="id", start=8, length=8)
    arg1 = Signal(start=16, length=16, name="arg1")
    arg2 = Signal(start=32, length=16, name="arg2")
    arg3 = Signal(start=48, length=16, name="arg3")

    cmd_args_msg.add_signal(dst)
    cmd_args_msg.add_signal(id)
    cmd_args_msg.add_signal(arg1)
    cmd_args_msg.add_signal(arg2)
    cmd_args_msg.add_signal(arg3)

    spec.common.add_msg(cmd_args_msg)

    cmd_return = Message(name="return_cmd", id=2)
    id = Signal(name="id", start=8, length=8)
    ret1 = Signal(start=16, length=16, name="ret1")
    ret2 = Signal(start=32, length=16, name="ret2")
    ret3 = Signal(start=48, length=16, name="ret3")

    cmd_return.add_signal(id)
    cmd_return.add_signal(ret1)
    cmd_return.add_signal(ret2)
    cmd_return.add_signal(ret3)

    spec.common.add_msg(cmd_return)


def default_common(spec):
    default_common_log_msg(spec)
    default_common_cfg_msg(spec)
    default_common_cmd_msg(spec)


def default_spec_logs(spec):
    log_error = Log(
        id=0, name="wrong_log_id", n_args=0, string="Log code was not found"
    )
    spec.add_log(log_error)

    cfg_error = Log(
        id=1, name="wrong_cfg_id", n_args=0, string="Cfg code was not found"
    )

    spec.add_log(cfg_error)


def default_spec(spec):
    default_spec_logs(spec)


def default_configs(spec):
    default_common(spec)

    default_spec(spec)
    for device in spec.devices.values():
        default_device(device)


def read_dbc(dbc, json_file, device_config, logger):
    # if device name configuration is provided load it
    if device_config:
        with open(device_config) as f:
            dev_config = json.loads(f.read())

    db = cantools.database.load_file(dbc)

    spec = Spec()

    for message in db.messages:
        logger.debug("db message " + str(message))
        sid = message.frame_id
        dev_id, msg_id = decompose_id(sid)

        # no device id configuration provided, dynamically ask for it
        if not device_config:
            device_name = query_user_ids(dev_id)
        else:
            device_name = dev_config[str(dev_id)]

        device = Device(id=dev_id, name=device_name, msgs={})

        msg = Message(name=message.name, id=msg_id, dlc=message.length, signals={})

        err = spec.add_device(device)
        logger.debug("add device " + str(dev_id) + " " + str(err) + " " + str(device))
        err = spec.devices[device.name].add_msg(msg)
        logger.debug("add message " + str(msg_id) + " " + str(err) + " " + str(msg))

        for signal in message.signals:
            t = find_type(signal)
            print(
                signal.multiplexer_signal, signal.is_multiplexer, signal.multiplexer_ids
            )
            sig = Signal(
                name=signal.name,
                start=signal.start,
                length=signal.length,
                scale=signal.scale or 1,
                offset=signal.offset or 0,
                unit=signal.unit or "",
                comment=signal.comment or "",
                min_value=signal.minimum or 0,
                max_value=signal.maximum or 0,
                byte_order=signal.byte_order or "little_endian",
                type=t or "unsigned",
                mux=signal.multiplexer_signal or "",
                mux_count=1,
            )
            err = spec.devices[device.name].get_msg(message.name).add_signal(sig)

            logger.debug(
                "add signal " + str(sig.name) + " " + str(err) + " " + str(sig)
            )

    default_configs(spec)
    j = spec.compile()
    # pprint(j)
    j["version"] = "0.2"

    with open(json_file, "w") as f:
        f.write(json.dumps(j, sort_keys=True, indent=4))


def init(json_file, logger):
    spec = Spec()
    default_configs(spec)
    logger.info("added default configs ✅")
    j = spec.compile()

    with open(json_file, "w") as f:
        f.write(json.dumps(j, sort_keys=True, indent=4))



import json
from fcp.spec import *
from fst_cantools import Message as CANmessage


def bitmask(n):
    return 2 ** n - 1


def encode_signal(signal, value):
    value = (value - signal.offset) / signal.scale
    return (int(value) & bitmask(signal.length)) << signal.start


def decode_signal(signal, value):
    value = (value >> signal.start) & bitmask(signal.length)

    if signal.type == 'signed':
        value = value ^ ((2 << signal.length) - 1)
    return (value * signal.scale) + signal.offset


class Fcp:
    def __init__(self, spec):
        self.messages = {}
        self.spec = spec

        for device in spec.devices.values():
            for msg in device.msgs.values():
                self.messages[msg.name] = msg

        for msg in spec.common.msgs.values():
            self.messages[msg.name] = msg

    def encode_msg(self, sid, msg_name, signals):
        msg = self.messages.get(msg_name)

        data = 0

        sigs = msg.signals

        for name, value in signals.items():
            signal = sigs[name]
            data |= encode_signal(signal, value)

        datas = [0, 0, 0, 0]
        for i in range(4):
            datas[i] = (data >> 16 * i) & 0xFFFF

        sid = make_sid(sid, msg.id)
        return CANmessage(sid, msg.dlc, datas, 1)

    def find_msg(self, msg):
        dev_id, msg_id = decompose_id(msg.sid)

        for dev in self.spec.devices.values():
            if dev.id == dev_id:
                for msg in dev.msgs.values():
                    if msg.id == msg_id:
                        return msg

        for msg in self.spec.common.msgs.values():
            if msg.id == msg_id:
                return msg

    def decode_msg(self, msg):
        fcp_msg = self.find_msg(msg)
        if fcp_msg == None:
            return "", {}

        signals = {}
        data = 0
        for i, d in enumerate(msg.data):
            data += d << 16 * i

        for name, signal in fcp_msg.signals.items():
            signals[name] = decode_signal(signal, data)

        return fcp_msg.name, signals



from cantools.database.can.database import Database as CanDatabase
from cantools.database.can.message import Message as CanMessage
from cantools.database.can.signal import Signal as CanSignal
from cantools.database.can.node import Node as CanNode

from cantools.database import *

from .spec import *


def is_float(signal):
    return signal.type == "float" or signal.type == "double"


def is_signed(signal):
    return signal.type == "signed"


def is_decimal(signal):
    return signal.type == "signed" or signal.type == "unsigned"


def is_multiplexer(signal, mux_signals):
    return signal.name in mux_signals


# multiplexer_signal=None,


def make_signal(signal, mux_signals):
    def make_signal_closure(signal, mux_signals, ids):
        return CanSignal(
            signal.name,
            signal.start,
            signal.length,
            byte_order=signal.byte_order,
            scale=signal.scale,
            offset=signal.offset,
            minimum=signal.min_value,
            maximum=signal.max_value,
            unit=signal.unit,
            comment=signal.comment,
            is_float=is_float(signal),
            is_signed=is_signed(signal),
            decimal=is_decimal(signal),
            is_multiplexer=is_multiplexer(signal, mux_signals),
            multiplexer_ids=ids,
            multiplexer_signal=[signal.mux],
        )

    if signal.mux_count == 1:
        yield make_signal_closure(signal, mux_signals, None)
        return

    for i in range(signal.mux_count):
        yield make_signal_closure(signal, mux_signals, [i])


def process_mux_signals(signals):
    mux_signals = []
    for key, val in signals.items():
        if val.mux in mux_signals:
            mux_signals.append(val.mux)

    return mux_signals


def write_dbc(j, dbc, logger):
    logger.info("Writing dbc file")

    spec = Spec()
    spec.decompile(j)

    messages = []
    nodes = []

    for dev_name, dev in spec.devices.items():
        nodes.append(CanNode(dev.name, ""))
        for msg_name, msg in dev.msgs.items():
            id = make_sid(dev.id, msg.id)

            mux_signals = process_mux_signals(msg.signals)
            signals = []
            for sig_name, sig in msg.signals.items():
                for sig in make_signal(sig, mux_signals):
                    signals.append(sig)

            messages.append(CanMessage(id, msg.name, msg.dlc, signals))

    db = CanDatabase(messages=messages, nodes=nodes)
    dump_file(db, dbc)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'signaldetails.ui',
# licensing of 'signaldetails.ui' applies.
#
# Created: Sun Nov  3 00:30:06 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_SignalDetails(object):
    def setupUi(self, SignalDetails):
        SignalDetails.setObjectName("SignalDetails")
        SignalDetails.resize(473, 580)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(SignalDetails)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(SignalDetails)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.nameLabel = QtWidgets.QLabel(self.groupBox)
        self.nameLabel.setObjectName("nameLabel")
        self.horizontalLayout_14.addWidget(self.nameLabel)
        self.nameEdit = QtWidgets.QLineEdit(self.groupBox)
        self.nameEdit.setObjectName("nameEdit")
        self.horizontalLayout_14.addWidget(self.nameEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_14)
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.commentLabel = QtWidgets.QLabel(self.groupBox)
        self.commentLabel.setObjectName("commentLabel")
        self.horizontalLayout_13.addWidget(self.commentLabel)
        self.commentEdit = QtWidgets.QLineEdit(self.groupBox)
        self.commentEdit.setObjectName("commentEdit")
        self.horizontalLayout_13.addWidget(self.commentEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_13)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.startLabel = QtWidgets.QLabel(self.groupBox)
        self.startLabel.setObjectName("startLabel")
        self.horizontalLayout_12.addWidget(self.startLabel)
        self.startEdit = QtWidgets.QLineEdit(self.groupBox)
        self.startEdit.setObjectName("startEdit")
        self.horizontalLayout_12.addWidget(self.startEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.lengthLabel = QtWidgets.QLabel(self.groupBox)
        self.lengthLabel.setObjectName("lengthLabel")
        self.horizontalLayout_11.addWidget(self.lengthLabel)
        self.lengthEdit = QtWidgets.QLineEdit(self.groupBox)
        self.lengthEdit.setObjectName("lengthEdit")
        self.horizontalLayout_11.addWidget(self.lengthEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_11)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.typeLabel = QtWidgets.QLabel(self.groupBox)
        self.typeLabel.setObjectName("typeLabel")
        self.horizontalLayout_9.addWidget(self.typeLabel)
        self.typeEdit = QtWidgets.QLineEdit(self.groupBox)
        self.typeEdit.setObjectName("typeEdit")
        self.horizontalLayout_9.addWidget(self.typeEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.unitLabel = QtWidgets.QLabel(self.groupBox)
        self.unitLabel.setObjectName("unitLabel")
        self.horizontalLayout_8.addWidget(self.unitLabel)
        self.unitEdit = QtWidgets.QLineEdit(self.groupBox)
        self.unitEdit.setObjectName("unitEdit")
        self.horizontalLayout_8.addWidget(self.unitEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.minValueLabel = QtWidgets.QLabel(self.groupBox)
        self.minValueLabel.setObjectName("minValueLabel")
        self.horizontalLayout_7.addWidget(self.minValueLabel)
        self.minValueEdit = QtWidgets.QLineEdit(self.groupBox)
        self.minValueEdit.setObjectName("minValueEdit")
        self.horizontalLayout_7.addWidget(self.minValueEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.maxValueLabel = QtWidgets.QLabel(self.groupBox)
        self.maxValueLabel.setObjectName("maxValueLabel")
        self.horizontalLayout_6.addWidget(self.maxValueLabel)
        self.maxValueEdit = QtWidgets.QLineEdit(self.groupBox)
        self.maxValueEdit.setObjectName("maxValueEdit")
        self.horizontalLayout_6.addWidget(self.maxValueEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.muxLabel = QtWidgets.QLabel(self.groupBox)
        self.muxLabel.setObjectName("muxLabel")
        self.horizontalLayout_5.addWidget(self.muxLabel)
        self.muxEdit = QtWidgets.QLineEdit(self.groupBox)
        self.muxEdit.setObjectName("muxEdit")
        self.horizontalLayout_5.addWidget(self.muxEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.muxCountLabel = QtWidgets.QLabel(self.groupBox)
        self.muxCountLabel.setObjectName("muxCountLabel")
        self.horizontalLayout_4.addWidget(self.muxCountLabel)
        self.muxCountEdit = QtWidgets.QLineEdit(self.groupBox)
        self.muxCountEdit.setObjectName("muxCountEdit")
        self.horizontalLayout_4.addWidget(self.muxCountEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.byteOrderLabel = QtWidgets.QLabel(self.groupBox)
        self.byteOrderLabel.setObjectName("byteOrderLabel")
        self.horizontalLayout_3.addWidget(self.byteOrderLabel)
        self.byteOrderEdit = QtWidgets.QLineEdit(self.groupBox)
        self.byteOrderEdit.setObjectName("byteOrderEdit")
        self.horizontalLayout_3.addWidget(self.byteOrderEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.scaleLabel = QtWidgets.QLabel(self.groupBox)
        self.scaleLabel.setObjectName("scaleLabel")
        self.horizontalLayout_2.addWidget(self.scaleLabel)
        self.scaleEdit = QtWidgets.QLineEdit(self.groupBox)
        self.scaleEdit.setObjectName("scaleEdit")
        self.horizontalLayout_2.addWidget(self.scaleEdit)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.offsetLabel = QtWidgets.QLabel(self.groupBox)
        self.offsetLabel.setObjectName("offsetLabel")
        self.horizontalLayout.addWidget(self.offsetLabel)
        self.offsetEdit = QtWidgets.QLineEdit(self.groupBox)
        self.offsetEdit.setObjectName("offsetEdit")
        self.horizontalLayout.addWidget(self.offsetEdit)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.saveSignal = QtWidgets.QPushButton(self.groupBox)
        self.saveSignal.setObjectName("saveSignal")
        self.horizontalLayout_10.addWidget(self.saveSignal)
        self.signalDeleteButton = QtWidgets.QPushButton(self.groupBox)
        self.signalDeleteButton.setObjectName("signalDeleteButton")
        self.horizontalLayout_10.addWidget(self.signalDeleteButton)
        self.verticalLayout.addLayout(self.horizontalLayout_10)
        self.verticalLayout_2.addWidget(self.groupBox)

        self.retranslateUi(SignalDetails)
        QtCore.QMetaObject.connectSlotsByName(SignalDetails)

    def retranslateUi(self, SignalDetails):
        SignalDetails.setWindowTitle(
            QtWidgets.QApplication.translate("SignalDetails", "SignalDetails", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("SignalDetails", "Signal", None, -1)
        )
        self.nameLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "name", None, -1)
        )
        self.commentLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "comment", None, -1)
        )
        self.startLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "start", None, -1)
        )
        self.lengthLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "length", None, -1)
        )
        self.typeLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "type", None, -1)
        )
        self.unitLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "unit", None, -1)
        )
        self.minValueLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "min_value", None, -1)
        )
        self.maxValueLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "max_value", None, -1)
        )
        self.muxLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "mux", None, -1)
        )
        self.muxCountLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "mux_count", None, -1)
        )
        self.byteOrderLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "byte_order", None, -1)
        )
        self.scaleLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "scale", None, -1)
        )
        self.offsetLabel.setText(
            QtWidgets.QApplication.translate("SignalDetails", "offset", None, -1)
        )
        self.saveSignal.setText(
            QtWidgets.QApplication.translate("SignalDetails", "Save", None, -1)
        )
        self.signalDeleteButton.setText(
            QtWidgets.QApplication.translate("SignalDetails", "Delete", None, -1)
        )



# Documentation generator
# Generate HTML from json
import os
from ww import f

from .docs_css import css


def markdown(spec, root):
    main = "% FCP Docs\n"
    # markdown += "# FCP Docs\n"
    main += f("## Logs\n")
    for log in sorted(spec.logs.values(), key=lambda x: x.id):
        main += f("* {log.id}: [{log.name}]({root}/log.md#{log.name})\n")

    main += "\n"

    main += f("## Devices\n")
    for dev in sorted(spec.devices.values(), key=lambda x: x.id):
        main += f("* [{dev.id}: {dev.name}]({root}/{dev.name.lower()}.md)\n")
        for msg in sorted(dev.msgs.values(), key=lambda x: x.id):
            main += f("  + [{msg.id}: {msg.name}]({root}/{dev.name.lower()}.md#{msg.name.lower()})\n")
    main += "\n"

    main += f("## Configs\n")
    for dev in sorted(spec.devices.values(), key=lambda x: x.id):
        main += f("* [{dev.id}: {dev.name}]({root}/{dev.name.lower()}.md)\n")
        for cfg in sorted(dev.cfgs.values(), key=lambda x: x.id):
            main += f("  + [{cfg.id}: {cfg.name}]({root}/{dev.name.lower()}.md#{cfg.name.lower()})\n")
    main += "\n"

    main += f("## Commands\n")
    for dev in sorted(spec.devices.values(), key=lambda x: x.id):
        main += f("* [{dev.id}: {dev.name}]({root}/{dev.name.lower()}.md)\n")
        for cmd in sorted(dev.cmds.values(), key=lambda x: x.id):
            main += f("  + [{cmd.id}: {cmd.name}]({root}/{dev.name.lower()}.md#{cmd.name.lower()})\n")
    main += "\n"

    devices = []
    for dev in spec.devices.values():
        device = f("% {dev.name} \n")
        device += f("[index]({root}/index.md)\n\n")
        device += "## Messages\n"
        for msg in sorted(dev.msgs.values(), key=lambda x: x.id):
            device += f("### {msg.id}. {msg.name}\n")
            device += f("* dlc: {msg.dlc}\n")
            device += f("* frequency: {msg.frequency}\n\n")
            for sig in sorted(msg.signals.values(), key=lambda x: x.start):
                device += f("#### {sig.name}\n")
                if sig.comment != "":
                    device += f("_{sig.comment}_\n\n")
                device += f("* start: {sig.start}\n")
                device += f("* length: {sig.length}\n")

                if sig.byte_order != "little_endian":
                    device += f("* byte_order: {sig.byte_order}\n")

                if not (sig.scale == 1 and sig.offset == 0):
                    device += f("* scale: {sig.scale}\n")
                    device += f("* offset: {sig.offset}\n")

                device += f("* type: {sig.type}\n")
                if sig.unit != "":
                    device += f("* unit: {sig.unit}\n\n")

                if not (sig.min_value == 0 and sig.max_value == 0):
                    device += f("* min_value: {sig.min_value}\n")
                    device += f("* max_value: {sig.max_value}\n")

                if not (sig.mux_count == 1 and sig.mux == ""):
                    device += f("* mux: {sig.mux}\n")
                    device += f("* mux_count: {sig.mux_count}\n")
                device += "\n"

        device += f("## Configs \n")
        for cfg in sorted(dev.cfgs.values(), key=lambda x: x.id):
            device += f("### {cfg.id}. {cfg.name}\n")
            if not cfg.comment == "":
                device += f("_{cfg.comment}_\n")
            device += "\n"

        device += f("## Commands \n")
        for cmd in sorted(dev.cmds.values(), key=lambda x: x.id):
            device += f("### {cmd.id}. {cmd.name}\n")
            if not cmd.comment == "":
                device += f("_{cmd.comment}_\n")
            device += "\n"

        devices.append(device)

    log_md = "% Logs\n"
    log_md += f("[index]({root}/index.md)\n\n")
    for log in sorted(spec.logs.values(), key=lambda x: x.id):
        log_md += f("## {log.name}\n")
        if log.comment != "":
            log_md += f("_{log.comment}_\n\n")

        log_md += f("* {log.string}\n")
        log_md += f("* {log.id}\n")
        log_md += f("* {log.n_args}\n")
        log_md += "\n"

    return main, log_md, devices


def check_out_dir(out):
    """Create output directory.

    :out: output directory path.
    :return: Failure and error message.
    """

    if not os.path.isdir(out):
        try:
            os.makedirs(out)
        except Exception as e:
            return False, str(e)

    return True, ""


def generate_docs(spec, out, link_location, logger):

    r, msg = check_out_dir(out)
    if not r:
        logger.error(msg)
        return

    root = link_location
    logger.info("Generate docs")

    build_sh = """
    for file in *.md; do 
            pandoc -s --css=pandoc.css -f markdown --to=html5 "$file" -o "$(basename "$file" .md).html" --lua-filter=links-to-html.lua; 
    done
    """

    links_to_html = """
-- links-to-html.lua
function Link(el)
  el.target = string.gsub(el.target, "%.md", ".html")
  return el
end
"""

    main, log_md, devices = markdown(spec, link_location)

    with open(os.path.join(out, "index.md"), "w") as _f:
        _f.write(main)

    for device, name in zip(devices, spec.devices.keys()):
        with open(os.path.join(out, name + ".md"), "w") as _f:
            _f.write(device)

    with open(os.path.join(out, "log.md"), "w") as _f:
        _f.write(log_md)

    with open(os.path.join(out, "build.sh"), "w") as _f:
        _f.write(build_sh)

    os.chmod(os.path.join(out, "build.sh"), 0o755)

    with open(os.path.join(out, "pandoc.css"), "w") as _f:
        _f.write(css)

    with open(os.path.join(out, "links-to-html.lua"), "w") as _f:
        _f.write(links_to_html)
    
    os.system(f("cd {out}; ./build.sh"))

    logger.info("Done")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'devicedetails.ui',
# licensing of 'devicedetails.ui' applies.
#
# Created: Sun Nov  3 00:30:05 2019
#      by: pyside2-uic  running on PySide2 5.13.1
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_DeviceDetails(object):
    def setupUi(self, DeviceDetails):
        DeviceDetails.setObjectName("DeviceDetails")
        DeviceDetails.resize(398, 382)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DeviceDetails)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.groupBox = QtWidgets.QGroupBox(DeviceDetails)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.nameLabel = QtWidgets.QLabel(self.groupBox)
        self.nameLabel.setObjectName("nameLabel")
        self.horizontalLayout.addWidget(self.nameLabel)
        self.nameEdit = QtWidgets.QLineEdit(self.groupBox)
        self.nameEdit.setObjectName("nameEdit")
        self.horizontalLayout.addWidget(self.nameEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.idLabel = QtWidgets.QLabel(self.groupBox)
        self.idLabel.setObjectName("idLabel")
        self.horizontalLayout_2.addWidget(self.idLabel)
        self.idEdit = QtWidgets.QLineEdit(self.groupBox)
        self.idEdit.setObjectName("idEdit")
        self.horizontalLayout_2.addWidget(self.idEdit)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.messageContents = QtWidgets.QWidget()
        self.messageContents.setGeometry(QtCore.QRect(0, 0, 334, 192))
        self.messageContents.setObjectName("messageContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.messageContents)
        self.verticalLayout.setObjectName("verticalLayout")
        self.scrollArea.setWidget(self.messageContents)
        self.verticalLayout_3.addWidget(self.scrollArea)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.cmdsButton = QtWidgets.QPushButton(self.groupBox)
        self.cmdsButton.setCheckable(True)
        self.cmdsButton.setObjectName("cmdsButton")
        self.horizontalLayout_5.addWidget(self.cmdsButton)
        self.cfgsButton = QtWidgets.QPushButton(self.groupBox)
        self.cfgsButton.setCheckable(True)
        self.cfgsButton.setObjectName("cfgsButton")
        self.horizontalLayout_5.addWidget(self.cfgsButton)
        self.verticalLayout_3.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.addButton = QtWidgets.QPushButton(self.groupBox)
        self.addButton.setObjectName("addButton")
        self.horizontalLayout_4.addWidget(self.addButton)
        self.deleteDeviceButton = QtWidgets.QPushButton(self.groupBox)
        self.deleteDeviceButton.setObjectName("deleteDeviceButton")
        self.horizontalLayout_4.addWidget(self.deleteDeviceButton)
        self.verticalLayout_3.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        self.cfgDetails = QtWidgets.QVBoxLayout()
        self.cfgDetails.setObjectName("cfgDetails")
        self.horizontalLayout_3.addLayout(self.cfgDetails)
        self.cmdDetails = QtWidgets.QVBoxLayout()
        self.cmdDetails.setObjectName("cmdDetails")
        self.horizontalLayout_3.addLayout(self.cmdDetails)
        self.verticalLayout_5.addLayout(self.horizontalLayout_3)
        self.verticalLayout_2.addWidget(self.groupBox)

        self.retranslateUi(DeviceDetails)
        QtCore.QMetaObject.connectSlotsByName(DeviceDetails)

    def retranslateUi(self, DeviceDetails):
        DeviceDetails.setWindowTitle(
            QtWidgets.QApplication.translate("DeviceDetails", "DeviceDetails", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("DeviceDetails", "Device", None, -1)
        )
        self.nameLabel.setText(
            QtWidgets.QApplication.translate("DeviceDetails", "name", None, -1)
        )
        self.idLabel.setText(
            QtWidgets.QApplication.translate("DeviceDetails", "id", None, -1)
        )
        self.cmdsButton.setText(
            QtWidgets.QApplication.translate("DeviceDetails", "Cmds", None, -1)
        )
        self.cfgsButton.setText(
            QtWidgets.QApplication.translate("DeviceDetails", "Cfgs", None, -1)
        )
        self.addButton.setText(
            QtWidgets.QApplication.translate("DeviceDetails", "Add", None, -1)
        )
        self.deleteDeviceButton.setText(
            QtWidgets.QApplication.translate("DeviceDetails", "Delete", None, -1)
        )



#!/usr/bin/env python3

"""c-generator.

Usage:
  c-generator.py generate <json> <template> <skel> <output>
  c-generator.py (-h | --help)
  c-generator.py --version

Options:
  -h --help     Show this screen.
  --version     Show version.
"""

import sys, os
import json

import logging
import shutil

from .spec import Spec
from .template import Tpl
from .generator import *
from .validator import validate


def check_output(output):
    if not os.path.isdir(output):
        try:
            os.makedirs(output)
        except Exception as e:
            return False, str(e)

    return True, ""


def write_files(output, can_ids, devices, common):
    def gen_output(d):
        return os.path.join(output, d)

    can_ids_c, can_ids_h = can_ids
    can_ids_path = gen_output("can_ids.h")
    with open(can_ids_path, "w") as f:
        f.write(can_ids_h)

    can_ids_path = gen_output("can_ids.c")
    with open(can_ids_path, "w") as f:
        f.write(can_ids_c)

    common_c, common_h = common
    common_c_path = gen_output("common.c")
    with open(common_c_path, "w") as f:
        f.write(common_c)

    common_h_path = gen_output("common.h")
    with open(common_h_path, "w") as f:
        f.write(common_h)

    for name, h, c in devices:
        c_path = gen_output(name + "_can.c")
        h_path = gen_output(name + "_can.h")

        with open(c_path, "w") as f:
            f.write(c)

        with open(h_path, "w") as f:
            f.write(h)


def check_version(j, logger):
    version = j.get("version")

    if version == None:
        logger.error("No version found")
        exit()

    if version != "0.2":
        logger.error("Wrong file version expected 0.2")
        exit()

    logger.info("Correct file version ✅")


def c_gen(templates, output, json_file, skel, logger):
    # copy skel directory
    status, err = check_output(output)
    for file in os.listdir(skel):
        shutil.copy(os.path.join(skel, file), output, follow_symlinks=True)

    if not status:
        logger.error(err)
        exit()

    tpl = Tpl(templates)
    status, err = tpl.check_tpl_dir()
    if not status:
        logger.error(err)
        exit()

    logger.info("Template dir checked ✅")

    tpl.open_tpl_files()

    with open(json_file) as f:
        r = f.read()
    # except Exception as e:
    #    logger.error(str(e))
    #    exit()

    logger.info("JSON spec file successfully read ✅")

    j = json.loads(r)
    ans = validate(logger, j)
    if not ans:
        return
    logger.info("Validated JSON ✅")
    spec = Spec()
    spec.decompile(j)

    check_version(j, logger)

    logger.info("JSON loaded into Spec ✅")

    can_ids = build_can_ids(spec, tpl)

    devices = [build_devices(spec, device, tpl) for device in spec.devices.values()]

    common = build_common(spec, tpl)

    write_files(output, can_ids, devices, common)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'cfgwidget.ui',
# licensing of 'cfgwidget.ui' applies.
#
# Created: Sat Dec  7 20:50:27 2019
#      by: pyside2-uic  running on PySide2 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_CfgWidget(object):
    def setupUi(self, CfgWidget):
        CfgWidget.setObjectName("CfgWidget")
        CfgWidget.resize(400, 300)
        CfgWidget.setMinimumSize(QtCore.QSize(300, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(CfgWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(CfgWidget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self._CfgScrollContents = QtWidgets.QWidget()
        self._CfgScrollContents.setGeometry(QtCore.QRect(0, 0, 356, 209))
        self._CfgScrollContents.setObjectName("_CfgScrollContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self._CfgScrollContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.CfgScrollContents = QtWidgets.QVBoxLayout()
        self.CfgScrollContents.setObjectName("CfgScrollContents")
        self.verticalLayout_2.addLayout(self.CfgScrollContents)
        self.scrollArea.setWidget(self._CfgScrollContents)
        self.verticalLayout_3.addWidget(self.scrollArea)
        self.addCfgButton = QtWidgets.QPushButton(self.groupBox)
        self.addCfgButton.setObjectName("addCfgButton")
        self.verticalLayout_3.addWidget(self.addCfgButton)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(CfgWidget)
        QtCore.QMetaObject.connectSlotsByName(CfgWidget)

    def retranslateUi(self, CfgWidget):
        CfgWidget.setWindowTitle(
            QtWidgets.QApplication.translate("CfgWidget", "CfgWidget", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("CfgWidget", "Configs", None, -1)
        )
        self.addCfgButton.setText(
            QtWidgets.QApplication.translate("CfgWidget", "Add Cfg", None, -1)
        )



"""c-generator.

Usage:
  validator.py validate <json>
  validator.py (-h | --help)
  validator.py --version

Options:
  -h --help     Show this screen.
  --version     Show version.
"""

import json
import logging

from ww import f

from .spec import *


def check_device(device):
    if len(device.msgs.keys()) == 0:
        return False, f("{device.name}: Device without messages")
    if device.id > 31:
        return False, f("{device.name}: dev_id bigger than 32")

    if len(device.name.strip()) != len(device.name):
        return False, f("{device.name}: Device name contains whitespace")

    return True, ""


def check_message(message):
    if len(message.signals.keys()) == 0:
        return False, f("{message.name}: Message without signals")

    if len(message.name.strip()) != len(message.name):
        return False, f("{message.name}: Message name contains whitespace")

    return True, ""


def check_device_message(message):
    r, msg = check_message(message)
    if not r:
        return r, msg

    if message.id > 63:
        return False, f("{message.name}: msg_id bigger than 64")
    # elif message.id < 32:
    #    return False, f"{message.name}: device has message id smaller than 32"

    return True, ""


def check_signal(key, signal):
    if signal.length > 64:
        return False, f("{signal.name}: signal length is bigger than 64")
    elif signal.length + signal.start > 64:
        return False, f("{signal.name}: signal outside 64 bit range")
    elif signal.start < 0:
        return False, f("{signal.name}: signals starts before bit 0")
    elif signal.length == 0:
        return False, f("{signal.name}: 0 length signal")
    elif not (
        signal.type == "unsigned"
        or signal.type == "signed"
        or signal.type == "double"
        or signal.type == "float"
    ):
        return (
            False,
            f("{signal.name}: Unsuported signal type, should be {unsigned, signed, double, float}",)
        )
    elif signal.type == "float" and signal.length != 32:
        return False, f("{signal.name}: float signal length is different than 32")
    elif signal.type == "double" and signal.length != 64:
        return False, f("{signal.name}: double signal length is different than 64")
    elif not (
        signal.byte_order == "little_endian" or signal.byte_order == "big_endian"
    ):
        return False, f("{signal.name}: signal endianess not suported")
    elif signal.name[0].isdigit():
        return False, f("{signal.name}: signal name starts with a number")
    elif signal.scale == 1 and signal.offset != 0:
        return (
            False,
            f("{signal.name}: signal offset is different than 0 for a scale of 1",)
        )
    elif signal.scale != 1 and signal.length > 32:
        return (
            False,
            f("{signal.name}: scaling for variables bigger than 32 bit is unsuported",)
        )
    elif key != signal.name:
        return False, f("{signal.name}: signal key is not equal to signal name")

    if len(signal.name.strip()) != len(signal.name):
        return False, f("{signal.name}: Device name contains whitespace")

    return True, ""


def check_common(message):
    r = check_message(message)
    if not r:
        return r

    if message.id > 63:
        return False, f("{message.name}: msg_id bigger than 64")
    elif message.id >= 32:
        return False, f("{message.name}: common has message id bigger or equal to 32")

    return True, ""


def check_log(log):
    print(log.id)
    if log.id < 0:
        return False, f("{log.name}: log id is smaller than 0")
    if log.id > 255:
        return False, f("{log.name} log id is bigger than 255")
    try:
        int(log.id)
    except Exception as e:
        return False, f("{log.name} is not a valid integer")

    return True, ""


def check_logs(logs):
    log_ids = []

    for log in logs.values():
        r, m = check_log(log)
        if r == False:
            return r, m

    for log in logs.values():
        if log.id in log_ids:
            return False, f("{log.name}: found duplicate log id")
        log_ids.append(log.id)

    return True, ""


def validate(logger, j, spec=None):
    if spec == None:
        spec = Spec()
        spec.decompile(j)

    r, msg = check_logs(spec.logs)
    if r == False:
        return r, msg

    for message in spec.common.msgs.values():
        r, msg = check_common(message)
        if not r:
            return r, msg

    device_ids = []
    device_names = []
    message_names = []
    signal_names = []
    for device in spec.devices.values():
        if device.id in device_ids:
            return False, f("{device.name}: duplicate device id")
        device_ids.append(device.id)

        if device.name in device_names:
            return False, f("{device.name}: duplicate device name")
        device_names.append(device.name)

        r, msg = check_device(device)
        if not r:
            return r, msg

        message_ids = []
        for message in device.msgs.values():
            if message.id in message_ids:
                return False, f("{device.name}: {message.name} duplicate message id")
            message_ids.append(message.id)

            if message.name in message_names:
                return False, f("{device.name}: {message.name} duplicate message name")
            message_names.append(message.name)

            if type(message) != Message:
                continue
            r, msg = check_device_message(message)
            if not r:
                return r, msg

            for key, signal in message.signals.items():
                if signal.name in signal_names:
                    return False, f("{message.name}: {signal.name} duplicate signal name")

                signal_names.append(signal.name)
                r, msg = check_signal(key, signal)
                if not r:
                    return r, msg

    return True, ""



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'logdetails.ui',
# licensing of 'logdetails.ui' applies.
#
# Created: Sat Dec  7 20:52:43 2019
#      by: pyside2-uic  running on PySide2 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_LogDetails(object):
    def setupUi(self, LogDetails):
        LogDetails.setObjectName("LogDetails")
        LogDetails.resize(400, 300)
        LogDetails.setMinimumSize(QtCore.QSize(200, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(LogDetails)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(LogDetails)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.nameLabel = QtWidgets.QLabel(self.groupBox)
        self.nameLabel.setObjectName("nameLabel")
        self.horizontalLayout_2.addWidget(self.nameLabel)
        self.nameEdit = QtWidgets.QLineEdit(self.groupBox)
        self.nameEdit.setObjectName("nameEdit")
        self.horizontalLayout_2.addWidget(self.nameEdit)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.idLabel = QtWidgets.QLabel(self.groupBox)
        self.idLabel.setObjectName("idLabel")
        self.horizontalLayout_3.addWidget(self.idLabel)
        self.idEdit = QtWidgets.QLineEdit(self.groupBox)
        self.idEdit.setObjectName("idEdit")
        self.horizontalLayout_3.addWidget(self.idEdit)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.n_argsLabel = QtWidgets.QLabel(self.groupBox)
        self.n_argsLabel.setObjectName("n_argsLabel")
        self.horizontalLayout_4.addWidget(self.n_argsLabel)
        self.n_argsEdit = QtWidgets.QLineEdit(self.groupBox)
        self.n_argsEdit.setObjectName("n_argsEdit")
        self.horizontalLayout_4.addWidget(self.n_argsEdit)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.commentLabel = QtWidgets.QLabel(self.groupBox)
        self.commentLabel.setObjectName("commentLabel")
        self.horizontalLayout_5.addWidget(self.commentLabel)
        self.commentEdit = QtWidgets.QLineEdit(self.groupBox)
        self.commentEdit.setObjectName("commentEdit")
        self.horizontalLayout_5.addWidget(self.commentEdit)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.stringLabel = QtWidgets.QLabel(self.groupBox)
        self.stringLabel.setObjectName("stringLabel")
        self.horizontalLayout.addWidget(self.stringLabel)
        self.stringEdit = QtWidgets.QLineEdit(self.groupBox)
        self.stringEdit.setObjectName("stringEdit")
        self.horizontalLayout.addWidget(self.stringEdit)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.logDeleteButton = QtWidgets.QPushButton(self.groupBox)
        self.logDeleteButton.setObjectName("logDeleteButton")
        self.verticalLayout_2.addWidget(self.logDeleteButton)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(LogDetails)
        QtCore.QMetaObject.connectSlotsByName(LogDetails)

    def retranslateUi(self, LogDetails):
        LogDetails.setWindowTitle(
            QtWidgets.QApplication.translate("LogDetails", "LogDetails", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("LogDetails", "Log", None, -1)
        )
        self.nameLabel.setText(
            QtWidgets.QApplication.translate("LogDetails", "name", None, -1)
        )
        self.idLabel.setText(
            QtWidgets.QApplication.translate("LogDetails", "id", None, -1)
        )
        self.n_argsLabel.setText(
            QtWidgets.QApplication.translate("LogDetails", "n_args", None, -1)
        )
        self.commentLabel.setText(
            QtWidgets.QApplication.translate("LogDetails", "comment", None, -1)
        )
        self.stringLabel.setText(
            QtWidgets.QApplication.translate("LogDetails", "string", None, -1)
        )
        self.logDeleteButton.setText(
            QtWidgets.QApplication.translate("LogDetails", "Delete", None, -1)
        )



import os
from jinja2 import Template

from ww import f


class Tpl:
    def __init__(self, template_dir):
        self.files = {}

        self.expected_files = [
            "h.jinja",
            "c.jinja",
            "can_ids_h.jinja",
            "can_ids_c.jinja",
            "common_c.jinja",
            "common_h.jinja",
        ]

        self.template_dir = template_dir
        self.tpl = {}
        return

    def check_tpl_dir(self):
        files = os.listdir(self.template_dir)

        for k in self.expected_files:
            if k not in files:
                return False, f("{k} missing")

        return True, ""

    def _get_tpl_dir_files(self):
        files = os.listdir(self.template_dir)
        expand_files = {}

        for k in self.expected_files:
            expand_files[k] = os.path.join(self.template_dir, k)

        return expand_files

    def open_tpl_files(self):
        self.files = self._get_tpl_dir_files()

        for k, v in self.files.items():
            with open(v) as f:
                self.tpl[k] = Template(f.read())



# from PySide2.QtWidgets import *
# from PySide2.QtGui import QKeySequence
# from PySide2.QtCore import Qt
#
# from .gui_lib.mainwindow import Ui_MainWindow
# from .gui_lib.devicewidget import Ui_DeviceWidget
# from .gui_lib.devicedetails import Ui_DeviceDetails
# from .gui_lib.messagewidget import Ui_MessageWidget
# from .gui_lib.messagedetails import Ui_MessageDetails
# from .gui_lib.signalwidget import Ui_SignalWidget
# from .gui_lib.signaldetails import Ui_SignalDetails
# from .gui_lib.logwidget import Ui_LogWidget
# from .gui_lib.logdetails import Ui_LogDetails
# from .gui_lib.cfgwidget import Ui_CfgWidget
# from .gui_lib.cfgdetails import Ui_CfgDetails
# from .gui_lib.cmdwidget import Ui_CmdWidget
# from .gui_lib.cmddetails import Ui_CmdDetails
# from .gui_lib.cmdarg import Ui_CmdArg
#
# def DetailsWidget(QWidget):
#    def __init__(self, gui, obj, parent):
#        QWidget.__init__(self)
#
#        self.atts = []
#
#    def save(self):
#        for att, get_f, set_f in self.atts:
#            set_f(att.text())
#
#        self.gui.reload()
#
#    def reload(self):
#        for att, get_f, set_f in self.atts:
#            att.setText(str(get_f()))
#
#    def delete(self):
#        r = self.gui.spec.rm_signal(self.signal)
#        self.setVisible(False)
#        self.dev_widget.setVisible(False)
#        return



from setuptools import setup, find_packages

setup(
    name="fcp",
    description="CAN bus manager",
    version="0.17",
    author="Joao Freitas",
    author_email="joaj.freitas@gmail.com",
    license="GPLv3",
    url="https://github.com/joajfreitas/fcp",
    download_url="https://github.com/joajfreitas/fcp/archive/v0.1.tar.gz",
    packages=find_packages(),
    entry_points={"console_scripts": ["fcp = fcp.__main__:main",],},
    install_requires=["click", "cantools", "PySide2", "jinja2", "ww"],
)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'logwidget.ui',
# licensing of 'logwidget.ui' applies.
#
# Created: Sat Dec  7 20:52:35 2019
#      by: pyside2-uic  running on PySide2 5.13.2
#
# WARNING! All changes made in this file will be lost!

from PySide2 import QtCore, QtGui, QtWidgets


class Ui_LogWidget(object):
    def setupUi(self, LogWidget):
        LogWidget.setObjectName("LogWidget")
        LogWidget.resize(400, 300)
        LogWidget.setMinimumSize(QtCore.QSize(300, 0))
        self.verticalLayout = QtWidgets.QVBoxLayout(LogWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(LogWidget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.LogScrollContents = QtWidgets.QWidget()
        self.LogScrollContents.setGeometry(QtCore.QRect(0, 0, 356, 209))
        self.LogScrollContents.setObjectName("LogScrollContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.LogScrollContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.logContents = QtWidgets.QVBoxLayout()
        self.logContents.setObjectName("logContents")
        self.verticalLayout_2.addLayout(self.logContents)
        self.scrollArea.setWidget(self.LogScrollContents)
        self.verticalLayout_3.addWidget(self.scrollArea)
        self.addLogButton = QtWidgets.QPushButton(self.groupBox)
        self.addLogButton.setObjectName("addLogButton")
        self.verticalLayout_3.addWidget(self.addLogButton)
        self.verticalLayout.addWidget(self.groupBox)

        self.retranslateUi(LogWidget)
        QtCore.QMetaObject.connectSlotsByName(LogWidget)

    def retranslateUi(self, LogWidget):
        LogWidget.setWindowTitle(
            QtWidgets.QApplication.translate("LogWidget", "LogWidget", None, -1)
        )
        self.groupBox.setTitle(
            QtWidgets.QApplication.translate("LogWidget", "Logs", None, -1)
        )
        self.addLogButton.setText(
            QtWidgets.QApplication.translate("LogWidget", "Add Log", None, -1)
        )
